/*
 *	_midithing.cc
 *
 *	Python extension main routines
 *
 */

#ifdef WITH_ALSA
#include <dlfcn.h>
#endif

#include "python-types.h"
#include "python-gtk-types.h"

/*
 *	defines for the module dict
 *
 */

typedef 
struct 
{
	int value;
	char * name; 
} 
m_define;

static m_define
m_event_defines[] =
{
	{0x80, M_NOTE_OFF_NAME},
	{0x90, M_NOTE_ON_NAME},
	{0xA0, M_NOTE_TOUCH_NAME},
	{0xB0, M_CONTROL_CHANGE_NAME},
	{0xC0, M_PROGRAM_CHANGE_NAME},
	{0xD0, M_CHANNEL_TOUCH_NAME},
	{0xE0, M_PITCH_WHEEL_NAME},
	{0xF0, M_SYSEX_NAME},
	{0xF7, M_EOX_NAME},
	{0xFF, M_META_NAME},

	{0, NULL}
};

static PyObject *
module_m_note_name (PyObject * self, PyObject * args)
{
	int note;
	char * name;

	if (PyArg_ParseTuple (args, "i", &note))
	{
		char buffer[32];
		m_note_event::note2string (note, buffer);
		
		return PyString_FromString (buffer);
	}
	else if (PyArg_ParseTuple (args, "s", &name))
	{
		PyErr_Clear();
		note = m_note_event::string2note (name);
		if (note < 0)
		{
			PyErr_SetString (PyExc_ValueError, "cannot interpret this as a note.");
			return NULL;
		}
		return PyInt_FromLong (note);
	}
	
	return NULL;
}

/*
 *	all methods the module contains. 
 */

static PyMethodDef 
_methods[] = 
{
	{M_NOTE_OFF_NAME, (PyCFunction) module_m_note_off_new, METH_VARARGS|METH_KEYWORDS, 
		"__doc__"},
	{M_NOTE_ON_NAME, (PyCFunction) module_m_note_new, METH_VARARGS|METH_KEYWORDS, 
		"__doc__"},
	{M_NOTE_TOUCH_NAME, (PyCFunction) module_m_note_touch_new, METH_VARARGS|METH_KEYWORDS, 
		"__doc__"},
	{M_CONTROL_CHANGE_NAME, (PyCFunction) module_m_controller_new, METH_VARARGS|METH_KEYWORDS, 
		"__doc__"},
	{M_PROGRAM_CHANGE_NAME, (PyCFunction) module_m_program_new, METH_VARARGS|METH_KEYWORDS, 
		"__doc__"},
	{M_CHANNEL_TOUCH_NAME, (PyCFunction) module_m_channel_touch_new, METH_VARARGS|METH_KEYWORDS, 
		"__doc__"},
	{M_PITCH_WHEEL_NAME, (PyCFunction) module_m_pitch_new, METH_VARARGS|METH_KEYWORDS, 
		"__doc__"},
	{M_SYSEX_NAME, (PyCFunction) module_m_sysex_new, METH_VARARGS|METH_KEYWORDS, 
		"__doc__"},
	{M_META_NAME, (PyCFunction) module_m_meta_new, METH_VARARGS|METH_KEYWORDS, 
		"__doc__"},

	{M_SONG_NAME, (PyCFunction) module_m_song_new, METH_VARARGS, 
		M_SONG_NAME" (['foo.mid'])\n "
		"returns a new "M_SONG_NAME". The filename is optional; if you "
		"specify it, "M_SONG_NAME"() tries to read that file."},
	{M_PART_NAME, (PyCFunction) module_m_part_new, METH_VARARGS|METH_KEYWORDS, 
		M_PART_NAME"([part,][event,][division=192])\n "
		"return a new "M_PART_NAME". Up to 254 "M_PART_NAME"s or "
		M_EVENT_NAME"s can be given; they will be copied into the new "
		M_PART_NAME" that is returned."},
	{M_EVENT_NAME, (PyCFunction) module_m_event_new, METH_VARARGS|METH_KEYWORDS, 
		M_EVENT_NAME" (status[, meta-type][, values and keywords])\n "
		"returns a new "M_EVENT_NAME". The status needs to be specified, and "
		"if it is "M_META_NAME", you'll have to specify a meta type. "
		"Parameters for the new "M_EVENT_NAME" can be specified with or without "
		"keywords."},
		
	{M_NOTE_NAME_NAME, (PyCFunction) module_m_note_name, METH_VARARGS|METH_KEYWORDS, 
		"Returns the name of a note."},

	#ifdef WITH_IO
	{M_LOOPBACK_NAME, (PyCFunction) module_m_loopback_new, METH_VARARGS, 
		M_LOOPBACK_NAME"()\n"
		"returns a pair of loopback ports; whatever you write to one will be written "
		"to the second. This is probably most useful for debugging purposes ;). "
	},
	{M_PORT_NAME, (PyCFunction) module_m_port_new, METH_VARARGS, 
		M_PORT_NAME" ('/path/to/port' [,'r' or 'w' or 'rw'])\n"
		"opens a MIDI port that is represented by the specified file. "
		"The file is opened read/write if the mode is omitted. "
		"It can subsequently be used by an "M_IO_NAME" object."
	},
	{M_ASCII_PORT_NAME, (PyCFunction) module_m_ascii_port_new, METH_VARARGS, 
		M_ASCII_PORT_NAME" (path or file descriptor)\n"
		"opens a port using the file you pass it. This "M_PORT_NAME
		" writes only Text Meta events. The file is opened write-only, in append "
		" mode. Other than this, the "M_ASCII_PORT_NAME" can be used like any "
		" other "M_PORT_NAME"."
	},
	#ifdef WITH_ALSA
	{M_ALSA_PORT_NAME, (PyCFunction) module_m_alsa_port_new, METH_VARARGS, 
		M_ALSA_PORT_NAME" ('alsa port name')\n"
		"opens a connection to an ALSA MIDI port and returns a "M_PORT_NAME" "
		"object. The read/write mode is determined automatically. Alternatively, "
		"you can call ALSAPort (client, port) where client and port are the "
		"numerical address instead of the name as printed by aconnect(1)."},
	#endif /* WITH_ALSA */
	{M_IO_NAME, (PyCFunction) module_m_io_new, METH_VARARGS, 
		M_IO_NAME"()\n "
		"returns a new "M_IO_NAME" object which is a list of "M_PORT_NAME" "
		"objects and can be used for output with a song."},
	{M_CLOCK_NAME, (PyCFunction) module_m_clock_new, METH_VARARGS, 
		M_CLOCK_NAME"()\n "
		"returns a new "M_CLOCK_NAME" that is used to drive a "M_SONG_NAME"."
		},
	#endif /* WITH_IO */

	#ifdef WITH_GTK
	{G_VIEW_NAME, (PyCFunction) module_gtk_view_new, METH_VARARGS,
		G_VIEW_NAME"()\n "
		"creates a "G_VIEW_NAME" widget."
		},
	#endif /* WITH_GTK */

	{NULL, NULL}
};


/*
 *	init_midithing
 *
 */
 
#ifdef DEBUG
FILE * __debug_fp = stderr;
#endif

extern "C" {

void
init_midithing()
{
	#ifdef DEBUG
	__debug_fp = fopen ("stderr", "a");
	if (!__debug_fp)
		__debug_fp = stderr;
	fcntl (fileno (__debug_fp), F_SETFL, 
		fcntl (fileno (__debug_fp), F_GETFL) | O_NONBLOCK);
		#ifdef DBG_C_REFSUM
		//atexit (c_ref_obj::print_refsum);
		#endif
	#endif

	#ifdef WITH_ALSA
	dlopen ("libasound.so", RTLD_LAZY | RTLD_GLOBAL);
	#endif
	
	#ifdef WITH_PYGTK
	pygtk_init();
	#endif

	PyObject * m = Py_InitModule4 
	(
		"_midithing", 
		_methods,
		"MIDI sequencing objects.",
		(PyObject *) NULL,
		PYTHON_API_VERSION
	);

	PyObject * d = PyModule_GetDict (m);

	m_define * def;
	def = m_event_defines;
	while (def->name)
	{
		c_char_buffer buffer;
		buffer << def->name << "Status";
		PyObject * string = PyString_FromStringAndSize (buffer.items, buffer.count);
		PyDict_SetItem (d, string, Py_BuildValue ("i", def->value));
		def++;
	}

	m_meta_type * meta_type = m_meta_event::types;
	for (int i = 0; i < 256; i++, meta_type++)
	{
		if (meta_type->name)
			PyDict_SetItemString (d, meta_type->name, Py_BuildValue ("i", i));
	}

	int ver, maj, min;
	if (sscanf (VERSION, "%d.%d.%d", &ver, &maj, &min))
		PyDict_SetItemString (d, "__version__", Py_BuildValue ("(iii)", ver, maj, min));

	PyDict_SetItemString (d, M_EVENT_NAME"Type", Py_BuildValue ("O", &m_event_py_type));
	PyDict_SetItemString (d, M_PART_NAME"Type", Py_BuildValue ("O", &m_part_py_type));
	PyDict_SetItemString (d, M_PORT_NAME"Type", Py_BuildValue ("O", &m_port_py_type));
	PyDict_SetItemString (d, M_SONG_NAME"Type", Py_BuildValue ("O", &m_song_py_type));
	#ifdef WITH_RTC
	PyDict_SetItemString (d, M_IO_NAME"Type", Py_BuildValue ("O", &m_io_py_type));
	#endif /* WITH_RTC */

	PyEval_InitThreads();
}

} /* extern "C" */

