/*
 *	arch-threads.h
 *
 *	thread, condition & semaphore abstraction layer.
 *
 *	Jul 30 2001 * added multiple reader / single writer lock.
 */


#ifndef _ARCH_THREADS_
#define _ARCH_THREADS_

#include <pthread.h>
#include <semaphore.h>

/*
 *	a_mutex
 * 
 */

class a_mutex
{
	public:
		enum {
			Fast = 0,
			Checking = 1,
			Recursive = 2
		};
		
		pthread_mutex_t	mutex;

#ifdef DEBUG
		int count;
#endif
						a_mutex (int kind = Recursive);
						~a_mutex();
						
		void		lock();
		void		unlock();
};

/*
 *	a_cond
 * 
 */

class a_cond
: public a_mutex
{
	public:
		pthread_cond_t	cond;

						a_cond();
						~a_cond();
						
		void		signal();
		void		wait();
		
		bool		timed_wait (double seconds);
};

/*
 *	a_semaphore
 *
 */

class a_semaphore
{
	public:
		sem_t		sem;

						a_semaphore();
						~a_semaphore();

		int			wait();
		int			try_wait();
		int			up();
		int			value();
};

/*
 *	a_thread
 * 
 */

class a_thread
{
	public:

		pthread_t			thread_id;
		bool					_thread_running;
		bool					_quit_requested;

									a_thread();
		virtual				~a_thread();
									
		bool					rt_thread_start (int priority);
		bool					thread_start();
		
		bool					thread_running() {return _thread_running;}

	private:
		static void	*	thread_init_func (a_thread * self);
		virtual void * thread() = 0;
		void					thread_exit (void * return_value);
};

/*
 * this code is adapted to c++ from the pthread-faq file:
 * 
 * $Id: rwlock.c,v 1.9 1999/02/27 14:19:35 lk Exp $
 *
 * Routines to implement a read-write lock. Multiple readers or one writer
 * can hold the lock at once. Writers are given priority over readers.
 * When compiled with RWLOCK_DEBUG defined verbose debugging output is
 * produced which can help track problems such as mismatches and
 * recursive locks.
 *
 * Copyright (C) 1998-99 Lee Kindness <leekindness@iname.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

class a_rwlock
{
	public:
    pthread_mutex_t lock;
    pthread_cond_t  rcond;
    pthread_cond_t  wcond;
    int             lock_count;
    int             waiting_writers;

										a_rwlock();
									 ~a_rwlock();

		void						lock_read();
		void						unlock_read();

		void						lock_write();
		void						unlock_write();

		static void			waiting_reader_cleanup (a_rwlock *);
		static void			waiting_writer_cleanup (a_rwlock *);
};


#endif
