/*
 *	common-out-buffer.cc
 *
 */

#include <unistd.h>
#include <string.h>
#include <errno.h>

#include "common-list.h"

/*
 *	c_char_buffer
 *
 */

c_char_buffer &
c_char_buffer::append (void * b, int len)
{
	allocate (len);
	memcpy (items + count, b, len);
	count += len;

	return *this;
}

/*
 *	append a string, literally, no \0 appended
 *
 */

c_char_buffer & 
c_char_buffer::operator << (char * s)
{
	static char null = '\0';

	int len;
	
	if (s != NULL)
		len = strlen (s);
	else
	{
		s = &null;
		len = 1;
	}
	
	return append (s, len);
}

/*
 *	append itoa of an int
 *
 */

c_char_buffer & 
c_char_buffer::operator << (int i)
{
	char buffer[32];
	sprintf (buffer, "%d", i);

	return *this << buffer;
}

/*
 *	append hex code of a byte
 *
 */

c_char_buffer & 
c_char_buffer::operator << (byte b)
{
	char buffer[3];
	sprintf (buffer, "%02x", b);

	return append (buffer, 2);
}

/*
 *	append %1.4f of a double
 *
 */

c_char_buffer & 
c_char_buffer::operator << (double d)
{
	char buffer[32];
	sprintf (buffer, "%1.4f", d);

	return *this << buffer;
}

/*
 *
 *
 */

c_char_buffer::c_char_buffer (int _granularity)
:
c_list<char> (_granularity)
{}

c_char_buffer::c_char_buffer (c_char_buffer * other)
:
c_list<char> (other->granularity)
{
	set (other->items, other->count);
}

void
c_char_buffer::bzero()
{
	memset (items, 0, count);
}

/*
 *	set data contents and don't ask questions
 *
 */

void
c_char_buffer::set (void * data, int length)
{
	if (length > allocated)
		allocate (length - allocated);
	if (length < allocated)
		deallocate (allocated - length);
		
	memcpy (items, data, length);
	count = length;
}

/*
 *	append another char buffer
 *
 */

c_char_buffer &
c_char_buffer::operator << (c_char_buffer * other)
{
	return append (other->items, other->count);
}

/*
 *	DBG operator[] with range check
 *
 */

#ifdef DEBUG
byte &
c_char_buffer::operator[] (int i)
{
	if (i < 0 || i > count)
	{
		_dbgprintf (2, "illegal c_char_buffer index %d\n", i);
		return *((byte *) items);
	}

	return *((byte *) items + i);
}
#endif

/*
 *	try to write the complete contents of the buffer.
 *
 */

int
c_out_buffer::write (int fd, int n_bytes)
{
	if (n_bytes < 0 || n_bytes > count - next_write)
		n_bytes = count - next_write;
		
	int n = ::write (fd, items + next_write, n_bytes);

	if (n != count)
		_dbgprintf (0, "[c_char_buffer::write] @ %d returned %d/%d (allocated=%d)\n", next_write, n, count, allocated);

	if (n == -1)
		_dbgprintf (1, "[c_char_buffer::write] returned %s\n", strerror (errno));

	if (n > 0)
	{
		next_write += n;
		if (next_write == count)
		{
			clear();
			next_write = 0;
		}
	}
	return count;
}
