/*
 *	list of items
 *
 */

#include <malloc.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>

#include "arch-types.h"
#include "common-list.h"

/*
 *	c_ref_list
 *
 *	a list of c_ref_obj'ects that are incref'd and decref'd on 
 *	insertion and removal. caution! NULL is supposed to be valid, too.
 */

c_ref_list::~c_ref_list()
{
	clear();
}

void
c_ref_list::clear()
{
	for (int i = 0; i < count; i++)
		C_XDECREF (items[i]);
	
	c_list<c_ref_obj *>::clear();
}

int
c_ref_list::find (c_ref_obj * o)
{
	for (int i = 0; i < count; i++)
		if (items[i] == o)
			return i;
	return -1;
}

int
c_ref_list::insert (c_ref_obj * o, int index)
{
	C_XINCREF (o);

	int i = c_list<c_ref_obj *>::insert (index);
	items[i] = o;

	return i;
}

bool
c_ref_list::remove (int i)
{
	if (i < 0 || i >= count)
		return false;

	C_XDECREF (items[i]);
	
	c_list<c_ref_obj *>::remove (i);
	
	return true;
}

bool
c_ref_list::remove (c_ref_obj * o)
{
	return remove (find (o));
}

bool
c_ref_list::replace (int i, c_ref_obj * o)
{
	if (i < 0 || i >= count)
		return false;
	
	C_XDECREF (items[i]);
	items[i] = o;
	C_XINCREF (items[i]);
	
	return true;
}

bool
c_ref_list::swap (int i, int j)
{
	if (i < 0 || i >= count)
		return false;
	if (j < 1 || j >= count)
		return false;
	
	c_ref_obj * o = items[i];
	items[i] = items[j];
	items[j] = o;

	return true;
}


