/*
 *	common-refcount.h
 *	
 *	reference counting base class
 *	
 */

#ifndef _COMMON_REFCOUNT_
#define _COMMON_REFCOUNT_

/* this is an arbitrary value. when this is an object's refcount,
 * the object has been created as part of another object, or on
 * the stack, so it must not be <inc|dec>ref'ed or deleted.
 */
#define NO_REFCOUNTING	-64382

#define DBG_C_XXXREF 0
#ifdef DEBUG
#define DBG_C_REFSUM
#endif

#define C_INCREF(x) \
	{ \
		if ((x)->_c_refcount != NO_REFCOUNTING) \
		{ \
			_dbgprintf (DBG_C_XXXREF, "<incref %s @ %p = %d>\n", (x)->_c_refname, (x), (x)->_c_refcount + 1); \
			(x)->incref(); \
		} \
	}
	
#define C_DECREF(x) \
	{ \
		if ((x)->_c_refcount != NO_REFCOUNTING) \
		{ \
			if ((x)->_c_refcount <= 0) BREAKPOINT; \
			_dbgprintf (DBG_C_XXXREF, "<decref %s @ %p = %d>\n", (x)->_c_refname, (x), (x)->_c_refcount - 1); \
			(x)->decref(); \
		} \
	}

#define C_XINCREF(x) {if (x) C_INCREF(x)}
#define C_XDECREF(x) {if (x) C_DECREF(x)}
	
class c_ref_obj
{
	public:
		int			_c_refcount;
		char *	_c_refname;
		
		#ifdef DBG_C_REFSUM
		static int _c_refsum;
		static void	print_refsum();
		#endif
							c_ref_obj ();
							c_ref_obj (char * name);

		virtual	 ~c_ref_obj(); 

		void			set_ref_name (char * name);
		
		void			disable_refcounting() {_c_refcount = NO_REFCOUNTING;}
		
		void			incref() 
							{
								if (_c_refcount == NO_REFCOUNTING)
									return;
								++_c_refcount;
								#ifdef DBG_C_REFSUM
								++_c_refsum;
								#endif
							}

		void			decref() 
							{
								if (_c_refcount == NO_REFCOUNTING)
									return;
								if (--_c_refcount == 0) 
									delete this; 
								#ifdef DBG_C_REFSUM
								--_c_refsum;
								#endif
							}
};

#endif
