#ifndef _C_RGB_CLASSES_
#define _C_RGB_CLASSES_

#include "arch-types.h"
#include "common-list.h"

/*
 *	let's call a rectangle a rectangle.
 *
 */

class c_rectangle
{
	public:
		int				x0, y0, 
							width, height;
		
							c_rectangle() {}
							c_rectangle (c_rectangle & other) {*this = other;}
							c_rectangle (int _x0, int _y0, int _width, int _height)
								{x0 = _x0; y0 = _y0, width = _width, height = _height;}
							
		void			set (int _x0, int _y0, int _width, int _height)
								{x0 = _x0; y0 = _y0, width = _width, height = _height;}
		
		c_rectangle & operator = (const c_rectangle & other)
								{	x0 = other.x0; y0 = other.y0;
									width = other.width; height = other.height;
									return *this; }

		bool			clip (c_rectangle & that);
		void			swallow (c_rectangle & that);
		void			move (int dx, int dy) {x0 += dx; y0 += dy;}
};

/*
 *	simple rgb color storage
 *
 */

class c_color
{
	public:
		int		r,g,b;

					c_color() {}
					c_color (unsigned int _packed) {set (_packed);}
					c_color (int _r, int _g, int _b) {set (_r, _g, _b);}
		
		void	set (unsigned int _packed)
						{r = _packed >> 16; g = (_packed >> 8) & 0xFF; b = _packed & 0xFF;}
		
		void	set (int _r, int _g, int _b)
						{r = _r, g = _g, b = _b;}
		
		c_color & operator = (const c_color & other)
						{set (other.r, other.g, other.b); return *this;}

		unsigned int packed() 
						{return (r << 16) | (g << 8) | b;}

		void	apply_alpha (int alpha)
						{r = (r + alpha) >> 1; g = (g + alpha) >> 1; b = (b + alpha) >> 1;}
};

/*
 *	and a list of c_colors
 *
 */

class c_palette
:
public c_list<c_color>
{
	public:
								c_palette();

		int					add (unsigned int packed);
};

/*
 *	a rgb pixel buffer
 *
 */

class c_rgb_buffer
{
	public:
		c_color	fore, back;

		byte *	buffer;
		int			rowstride, pixelstride;
		int			width, height;

						c_rgb_buffer (int _width, int _height, int _pixelstride = 3);
					 ~c_rgb_buffer();

		bool		resize (int _width, int _height);
		bool		allocate();
		
		void		scroll (int dx, int dy);

		byte *	line_ptr (int y)
							{return buffer + y * rowstride;}
		byte *	pixel_ptr (int x, int y)
							{return buffer + y * rowstride + x * pixelstride;}
	
		void		rect (c_rectangle * r, c_color * c);
		void		rect (int x0, int y0, int width, int height, c_color * c);

		void		arect (c_rectangle * r, c_color * c, int alpha);
		void		arect (int x0, int y0, int width, int height, c_color * c, int alpha);
		
		void		arect_with_cut (c_rectangle * r, c_color * c, int alpha);

		void		apixel (int x, int y, c_color * c, int alpha);

		void		aaline (int x0, int y0, int x1, int y1, c_color * c);
};

#endif
