#! /usr/bin/env python
#
#	yet another configure script.

import sys, os, re
from distutils.sysconfig import get_config_vars

class Option:
	def __init__ (self, name, with, help, path):
		self.name = name
		self.with = with
		self.help = help
		self.path = path
	def __call__ (self, name):
		global options
		for option in options:
			if option.name == name:
				return option
		raise NameError
		
options = [
	Option ('alsa', 1, 'include support for the ALSA MIDI system (alsa >= 0.9.0b8)', None),
	Option ('debug', 0, 'make a debug build that dumps info into \'./stderr\'', None),
	Option ('flats', 0, 'print flat notes instead of sharps by default', None),
	Option ('rtc', 1, 'include support for /dev/rtc (linux/i386 or alpha)', None),
	Option ('io', 1, 'include the sequencer', None),
	Option ('gtk', 1, 'include the gtk event viewer', None),
	Option ('pygtk', 1, 'include PyGTK support (--with-pygtk=/path/to/pygtk)', None)
]

py_vars = get_config_vars()

VERSION			= ['0.4.2']
DEFINES_IN	= ['-DVERSION=\\"' + VERSION[0] + '\\"',]
CFLAGS_IN		= ['-Wall', '-g']
INCLUDES		= ['-I' + py_vars['INCLUDEPY']]
LIBS				= ["-rdynamic", "-L/usr/lib", "-L/usr/local/lib", 
							 "-L.", py_vars['LIBS'], "-lpthread"]

def usage():
	print """
Usage: """ + sys.argv[0] + """ [--with-option --without-option ... ]
		
  where option is one of\n""",
	
	for option in options:
		if option.with: 
			with = " [yes] "
		else: 
			with = " [ no] "
		
		print "  " + option.name.rjust(12) + with + option.help
	print
	# done

def shell (command):
	print "running", command
	args = ["/bin/sh", "-c", command]
	os.spawnv (os.P_WAIT, '/bin/sh', args)

def get_uname_processor():
	uname_w, uname_r = os.popen2 ('uname -m')
	return uname_r.read().strip()

def get_uname_osname():
	uname_w, uname_r = os.popen2 ('uname -s')
	return uname_r.read().strip()

def get_uname_release():
	uname_w, uname_r = os.popen2 ('uname -r')
	strings = uname_r.read().strip().split('.')
	release = []
	for r in strings:
		release.append ([int (r)])
	return release
	
def print_list (file, name, list): 
	# Makefile variables writer
	if not len (list):
		return
	file.write (name + " = ")
	comma = ''
	for i in list:
		file.write (comma + i)
		comma = ' \\\n\t'
	file.write ('\n\n')

def check_dependencies():
	if option('gtk').with <= 0:
		option ('pygtk').with = 0

	if option('io').with <= 0:
		option('rtc').with = 0
		option('alsa').with = 0



if not ('thread' in sys.builtin_module_names):
	print """
*Fatal*

Your installation of Python does not support threads.
"""
	sys.exit (1)




for option in options:
	exp = re.compile ('^--with(out)?-' + option.name + '(=.*)?')
	for arg in sys.argv[1:]:
		match = exp.match (arg)
		if not match:
			continue
		groups = match.groups()
		if groups[0] == 'out':
			option.with = 0
		elif groups[1]:
			option.with = 1
			option.path = groups[1][1:]
		else:
			option.with = 1

check_dependencies()

if option('io').with:
	if option('rtc').with:
		print "checking for RTC support ...",
		try:
			s = os.stat ('/usr/include/linux/rtc.h')
			DEFINES_IN.append ('-DHAVE_RTC_H')
			print "yes."
			option('rtc').with = 1
		except:
			try:
				s = os.stat ('/usr/include/linux/mc146818rtc.h')
				DEFINES_IN.append ('-DHAVE_MC146818RTC_H')
				print "yes."
				option('rtc').with = 1
			except:
				print "no."
				option('rtc').with = -1
			
	if option('alsa').with:
		print "checking for ALSA support ...",
		try:
			s = os.stat ('/usr/include/alsa/asoundlib.h')
			DEFINES_IN.append ('-DHAVE_ASOUNDLIB_H')
			print "yes."
			option('alsa').with = 1
		except:
			print "no."
			option('alsa').with = -1

if option('gtk').with:
	print "checking for GTK+ support ...",
	try:
		gtk_w, gtk_r = os.popen2 ('gtk-config --version')
		gtk_version = gtk_r.read().strip()
		major, minor, micro = gtk_version.split('.')
		print "yes, version", gtk_version
		if major < 1 or (major == 1 and minor < 2):
			print "GTK (" + gtk_version + ") should be 1.2.x. ",
			raise ValueError
			
		gtk_w, gtk_r = os.popen2 ('gtk-config --cflags')
		for string in gtk_r.read().strip().split(' '):
			INCLUDES.append (string)
			
		gtk_w, gtk_r = os.popen2 ('gtk-config --libs')
		for string in gtk_r.read().strip().split(' '):
			LIBS.append (string)
	except:
		print "no."
		option('pygtk').with = -1
		option('gtk').with = -1

if option('pygtk').with:
	print "checking for PyGTK+ support ...",
	try:
		if option ('pygtk').path:
			os.stat (option ('pygtk').path + '/pygtk.h')
		else:
			try:
				os.stat ('/usr/include/pygtk/pygtk.h')
			except:
				os.stat ('/usr/local/include/pygtk/pygtk.h')
		print "yes."
	except:
		print "can't find pygtk.h in", option ('pygtk').path, "or /usr/local/include/pygtk/"
		option ('pygtk').with = -1

if '--help' in sys.argv or '-h' in sys.argv:
	usage()
	sys.exit(0)


if '--show' in sys.argv:
	for v in py_vars.keys():
		print v, ":", py_vars[v]
	sys.exit (0)




py_dest = py_vars ['LIBDEST'] + '/site-packages'
if not py_dest in sys.path:
	print py_dest, "is not in sys.path. bailing out."
	sys.exit (1)

PY_DEST = [py_dest]


if option('pygtk').with:
	if option ('pygtk').path:
		INCLUDES.append ('-I' + option ('pygtk').path)

if option('alsa').with:
	LIBS.append ('-lasound')

if option('debug').with:
	DEFINES_IN.append ('-DDEBUG')
	print """
*Note*

You have selected to compile with debugging info. Midithing will write
debug output to a file called ./stderr, or, if that file cannot be written
to, stderr itself (usually your terminal).
"""
else:
	CFLAGS_IN.append ('-O2')

for option in options:
	if option.with:
		DEFINES_IN.append ('-DWITH_' + option.name.upper())




# make Makefile 

lines = open ('Makefile.in').readlines()

print "writing Makefile"
file = open ('Makefile', 'w')
thru = 1
for line in lines:
	if line.find ('# configure.py start #') == 0:
		file.write (line)
		print >> file, "# generated by configure.py"
		print >> file, "# " + " ".join (sys.argv[1:])
		file.write ('\n')
		thru = 0
		for var in ('VERSION', 'CFLAGS_IN', 'INCLUDES', 'DEFINES_IN', 'LIBS', 'PY_DEST'):
			print_list (file, var, eval (var))
			
	elif line.find ('# configure.py end #') == 0:
		thru = 1
	if thru:
		file.write (line)
file.close()

shell ("rm *.d > /dev/null 2>&1")

print "'make' should now make midithing version", VERSION[0]
