/*
 *	gtk-event-view.h
 *
 */

#ifdef WITH_GTK

#ifndef _GTK_EVENT_VIEW_
#define _GTK_EVENT_VIEW_

#include <gtk/gtk.h>

#include "midi-xy-mapper.h"
#include "common-refcount.h"
#include "common-rgb-classes.h"

/*
 *	draws events in a gtk drawing area widget
 *
 */

class gtk_event_view
:
public c_ref_obj
{
	public:
		m_song					song;
		m_part					selection;

		struct {
			c_rectangle 		size;
			GdkPixmap *			map;
			c_rgb_buffer *	buffer;
		} pixels;

		struct {
			int							button_mask;
			int							x, y;
			int							dz;
			guint						xtick;
		} clicked;
		
		struct {
			int							button_mask;
			int							x, y;
			int							dz;
			guint						xtick;
		} motion;
		
		struct {
			int							dx, dy;
		} scroll_pending;

		struct {
			m_song *				song;
			bool						redraw;
			m_tick					tick;
			int							x;
		} follow;

		int							magnification;
		m_tick					snap;

		m_x_mapper			x_mapper;
		m_y_mapper			y_mapper;

		c_list<c_rectangle> exposures;

		c_list<int>			velocity_to_alpha;

		c_color					fore;
		c_color					staff;
		c_color					back;
		c_color					loop;

		int							staff_alpha;

		c_palette				colors;
		
		guint						timeout_id;

		GtkWidget *			widget;
		GdkGC *					gc;

		guint						periodcounter;

										gtk_event_view (GtkWidget * _widget = NULL); /* can be made from a existing DrawingArea */
									 ~gtk_event_view();
		
		bool						scroll (int dx, int dy = 0);
		void						delayed_scroll (int dx, int dy = 0);
		void						periodic();

		void						full_redraw();

		void						redraw (int x0, int y0, int dx, int dy);
		void						redraw (c_rectangle * r); 

		void						blit (int x0, int y0, int dx, int dy);
		void						blit (c_rectangle * r) 
											{blit (r->x0, r->y0, r->width, r->height);}
		
		bool						event_rectangle (m_event * e, c_rectangle * r);
		
		c_color *				get_color (int port, int channel);
		void						draw_note (m_event * e, c_color * color, c_rectangle * clip);
		
		void						draw_click (m_tick tick, int alpha, c_rectangle * clip);
		void						draw_stafflines (c_rectangle * clip);
		void						draw_loop_markers (c_rectangle * clip);

		int							process_exposures();

		void						set_follow (m_song * _song = NULL);
		void						set_magnification (int _magnification, int x = -1, int y = -1);

		unsigned int		get_xwindow();

		static gint			delete_event (GtkWidget *, GdkEvent *, gtk_event_view *);
		static gint			expose_event (GtkWidget *, GdkEventExpose *, gtk_event_view *);
		static gint			configure_event (GtkWidget *, GdkEventConfigure *, gtk_event_view *);
		static gint			key_event (GtkWidget *, GdkEventKey *, gtk_event_view *);
		static gint			button_event (GtkWidget *, GdkEventButton *, gtk_event_view *);
		static gint			motion_event (GtkWidget *, GdkEventMotion *, gtk_event_view *);
		static gint			timeout_event (gtk_event_view *);
		
		void						init_colors();

		void						show_position();
		void						hide_position();
		void						update_position();
};

/*
 *	event drawing helper
 *
 */

class gtk_event_drawer
:
public m_connector
{
	public:
		gtk_event_view *	view;
		c_rectangle				clip;
		m_tick						bar_snap, beat_snap, fraction_snap;

		struct {
			m_tick	start, end;
		} tick;
											gtk_event_drawer (gtk_event_view *, c_rectangle *);
										
		virtual bool			event_in (m_event *, m_event_stamp * = 0);
		virtual void			click_in (m_tick tick, int bar, int beat, int tick);
};

class gtk_event_selector
:
public m_connector
{
	public:
		gtk_event_view *	view;
		c_rectangle				clip;

											gtk_event_selector (gtk_event_view *, c_rectangle *);
										
		virtual bool			event_in (m_event *, m_event_stamp * = 0);
		virtual void			click_in (m_tick tick, int bar, int beat, int tick);
};

#endif /* _GTK_EVENT_VIEW_ */

#endif /* WITH_GTK */
