"""The Flying Circus."""

import thread, sys

from midithing import *
from gtk import *
import GTK
import pixmaps
from inputs import *
from partlist import *
#from controls import *
from widgets import *

class StateVar:
	"""A property of the main thing"""
	def __init__ (self, value):
		self.name = 'unknown'
		self.value = value

	def set_value (self, value, widget = None):
		print "state[%s].set_value (%s): you should never get here!?" % (self.name, value)


class Pygmy:
	"""For want of a better name. This is the main windowing object."""
	def __init__ (self):
		self.song = None
		
		self.audible = Song()

		self.view = EventView()
		self.visible = self.view.song
		self.view.follow = self.audible
		
		self.io = IO()
		self.io.on_input = self.on_input
		self.io.sync.add (self.audible)

		self.init_window()

		self.init_state()


	def set_song (self, song):
		"""Set the song this pygmy handles."""
		if not song or not song.name:
			self.window.set_title (__name__)
		else:
			self.window.set_title (song.name + " - " + __name__)
		
		self.song = song
		self.audible.clear()
		self.visible.clear()

		if not song:
			self.view.redraw()
			return

		self.audible.loop = self.song.loop

		self.visible.division = \
		self.audible.division = \
		self.view.snap = \
		self.snap = song.division
			
		for part in song:
			part.name = part.name or "part %d" % song.index (part)
			self.audible.add (part)
			self.visible.add (part)

		self.view.redraw()
		self.widget ('part').set_song (song)
		self.widget ('tick').update_label()


	def init_window (self):
		"""Creates the window and all of its widgets; takes care of connecting
			 the widgets to the appropriate signal methods like change_*, sig_*
			 and write_*."""
		self.window = GtkWindow()
		self.window.set_title (__name__)
		
		box = GtkVBox()
		self.window.add (box)
		
		# three boxes in the main window
		self.top_box = GtkHBox()
		box.pack_start (self.top_box, expand = 1)
		
		self.mid_box = GtkHBox()
		box.pack_start (self.mid_box, expand = 0)

		self.bottom_box = GtkVBox()
		box.pack_start (self.bottom_box, expand = 0)
		
		# fills the left box
		# a button to hide the left toolbox (sig_left_visible)
		button = GrayDotButton (self, 'left_visible')
		self.top_box.pack_start (button, expand = 0)

		tools = (
			(PartList (self, 'part'),), 
			(ButtonInput (self, 'new'), ButtonInput (self, 'delete')),
		)

		self.left_table = GtkTable (len (tools), 6)
		self.left_frame = Gtk_Frame (self.left_table, GTK.SHADOW_OUT)
		self.top_box.pack_start (self.left_frame, expand = 0)
		
		self.left_table.set_border_width (4)
		
		r = 0 # row
		for row in tools:
			cols = (6, 6, 3, 2) [len (row)]
			c = 0
			for widget in row:
				widget = widget or GtkLabel (' ')
				xoptions = GTK.FILL
				yoptions = not r and GTK.EXPAND | GTK.FILL
				self.left_table.attach (widget, c, c + cols, r, r + 1, xoptions, yoptions)
				self.connect_widget (widget)
				c += cols
			r = r + 1
		


		# the middle row
		
		tools = (
			(
				None,
				(XPMToggleInput (self, 'record'), 64),
				(XPMToggleInput (self, 'playing'), 64),
				(SongPositionInput (self, 'tick', self.audible)),
				None
			),
		)

		self.mid_table = GtkTable (len (tools), 6)
		self.mid_frame = Gtk_Frame (self.mid_table, GTK.SHADOW_OUT)
		self.mid_box.pack_start (self.mid_frame, expand = 1)
		
		self.mid_table.set_border_width (4)
		
		r = 0 # row
		tupletype = type (tools)
		for row in tools:
			c = 0
			for object in row:
				xoptions = GTK.FILL
				yoptions = GTK.EXPAND | GTK.FILL
				if type (object) == tupletype:
					(widget, width) = object
					widget['width'] = width
				elif object:
					widget = object
				else:
					widget = GtkLabel (' ')
					xoptions |= GTK.EXPAND
				self.mid_table.attach (widget, c, c + 1, r, r + 1, xoptions, yoptions)
				self.connect_widget (widget)
				c += 1
			r = r + 1
		

		# the bottom box

		self.view_widget = \
		widget = GtkDrawingArea (self.view.widget)
		
		widget.connect ('motion-notify-event', self.sig_view_focussed)
		self.top_box.pack_start (widget, expand = 1)
		
		# a button to hide the bottom toolbox (sig_tools_visible)

		button = GrayDotButton (self, 'tools_visible')
		self.bottom_box.pack_end (button, expand = 0)

		self.tool_box = GtkHBox()
		self.tool_frame = Gtk_Frame (self.tool_box, GTK.SHADOW_OUT)
		self.bottom_box.add (self.tool_frame, expand = 0)
		

		tools = (
			(	
				(ColorInput (self, 'color'), 2),
				(ButtonInput (self, 'write_channel', '#ff0000'), 2), 
				ParameterInput (self, 'port', 0, 4)
			),
			(None, None, ParameterInput (self, 'channel', 0, 16)),
			(None, None, None),
			(	
				(ToggleInput (self, 'live_program', '#00ff00'), 2),
				(ButtonInput (self, 'write_program', '#ff0000'), 2), 
				BankInput (self, 'bank', 0, 4)
			), 
			(None, None, ParameterInput (self, 'program', 0, 128)),
      (None, None, None),
			(	
				ToggleInput (self, 'live_volume', '#00ff00'),
				ButtonInput (self, 'write_volume', '#ff0000'),
				ParameterInput (self, 'volume', 0, 128)
			), 
      (None, None, None),
			(	
				None, 
				(ButtonInput (self, 'write_note', '#ff0000'), 2),
				NoteInput (self, 'note')
			),
			(None, None, VelocityInput (self, 'velocity')),
			(	
				None,
				ButtonInput (self, 'write_note_off', '#ff0000'),
				ParameterInput (self, 'velocity_off', 0, 128)
			), 
      (None, None, None),
			(	
				ToggleInput (self, 'live_pitch', '#00ff00'), 
				ButtonInput (self, 'write_pitch', '#ff0000'),
				PitchInput (self, 'pitch')
			),
      (None, None, None),
			(	
				(ToggleInput (self, 'live_controller', '#00ff00'), 2), 
				(ButtonInput (self, 'write_controller', '#ff0000'), 2),
				ParameterInput (self, 'controller', 0, 128)
			),
			(None, None, ParameterInput (self, 'value', 0, 128)),
		)

		self.tool_table = GtkTable (3, len (tools))
		self.tool_box.pack_start (self.tool_table, expand = 1)
		
		self.tool_table.set_border_width (8)
		self.tool_table.set_row_spacing (0, 2)
		self.tool_table.set_row_spacing (1, 2)
		
		
		col_width = (2, 1, 1)
		xoptions = (GTK.FILL, GTK.FILL, GTK.FILL)
		tupletype = type ((None,))
		c = 0 # column
		for strip in tools:
			if strip == (None, None, None):
				# we just need something that is blank as a spacer
				widget = GtkLabel (' ') 
				self.tool_table.attach (widget, c, c + 1, 0, 4, xoptions = GTK.EXPAND|GTK.FILL)
			r = 0
			for object in strip:
				widget = None
				if type (object) == tupletype:
					widget, cols = object
					self.tool_table.attach (widget, c, c + cols, r, r + 1, xoptions[r])
					self.connect_widget (widget)
				elif object:
					self.tool_table.attach (object, c, c + 1, r, r + 1, xoptions[r])
					self.connect_widget (object)
				r += 1
			c += 1

		# this is the minimum size.
		self.window.set_usize (1200, 500)
		self.window.show_all()
		# acceptable hints are min_width, min_height, max_width,
		# max_height, base_width, base_height, width_inc, height_inc
		# (with int values) and min_aspect, max_aspect (double values).
		self.window.set_geometry_hints (None, min_width = 400, min_height = 400)
		self.current_pitch = 0.0
		self.period_counter = 1.0
		self.timeout = timeout_add (40, self.sig_periodic)

	def connect_widget (self, widget):
		"""Tries to connect a [Input] widget to the corresponding method of self.
			 Currently, we have three signal slots (methods) for 'x':
			 * 'change_x' changes a state variable,
			 * 'sig_x' triggers an action, and 
			 * x == 'write_y' causes a MIDI event sequence to be written."""
		try: name = widget.name
		except: return
		# this is a special case:
		if name == 'write_note':
			widget.button.connect ('pressed', self.write_note)
			widget.button.connect ('released', self.write_note_off)
			return
		method = \
			getattr (self, 'change_' + name, None) or \
			getattr (self, 'sig_' + name, None) or \
			(name[:6] == 'write_' and getattr (self, name, None))
		widget.emit_signal = method
	
	# Pygmy signal functions
	
	def sig_new (self, *args):
		"""makes a new part"""
		part = Part()
		part.port = self.port
		part.channel = self.channel
		part.name = "part %d" % len (self.song)
		
		self.song.add (part)
		self.audible.add (part)
		self.visible.add (part)
		self.widget ('part').add_part (part)

	def sig_delete (self, *args):
		if not self.part: return
		if not self.song.index (self.part): return

		if self.part in self.audible:
			self.audible.remove (self.part)
		if self.part in self.visible:
			self.visible.remove (self.part)
			if len (self.part): self.view.redraw()
		
		self.song.remove (self.part)
		self.widget ('part').remove_part (self.part)

	def toggle_audible (self, part):
		if part in self.audible: 
			self.audible.remove (part); return 0
		else:
			self.audible.add (part); return 1

	def toggle_visible (self, part):
		if part in self.visible: 
			self.visible.remove (part); return 0
		else:
			self.visible.add (part); return 1

	def sig_view_focussed (self, *args):
		self.change_focus (self.view)

	def sig_tools_visible (self, *args):
		self.tool_frame ['visible'] = not self.tool_frame ['visible']
		self.bottom_box.queue_resize()
		
	def sig_left_visible (self, *args):
		self.left_frame ['visible'] = not self.left_frame ['visible']
		self.top_box.queue_resize()
		
	def sig_periodic (self, *args):
		if self.audible.playing:
			self.widget ('tick').update_label()

		if self.live_pitch:
			return 1
		if self.pitch != self.current_pitch:
			self.change_pitch (self.current_pitch)
			
		if self.pitch:
			if self.focus == self.view:
				self.focus.move (self.pitch / 15)
			elif hasattr (self.focus, 'move'):
				self.period_counter /= 2.0
				if self.period_counter < abs (self.pitch):
					if self.focus.move (self.pitch) and hasattr (self, 'click_on'):
						# write a click as feedback
						self.io.write (self.click_on)
					self.period_counter = 1.0
					
		return 1 # we want more signals

	# Pygmy state
	
	def init_state (self):
		self.state = {
			'snap' :						StateVar (192),
			
			'color' :						StateVar ((0xfe, 0xf7, 0xe3)),

			'port' :						StateVar (0),
			'channel' :					StateVar (0),
			
			'volume' :					StateVar (90),
			
			'bank' :						StateVar (0),
			'program' :					StateVar (0),
			
			'note' :						StateVar (60),
			'velocity' :				StateVar (70),
			'velocity_off' :		StateVar (0),
			
			'pitch' :						StateVar (0.0),
			
			'controller' :			StateVar (0),
			'value' :						StateVar (0),

			'live_program' :		StateVar (1),
			'live_volume' :			StateVar (1),
			'live_pitch' :			StateVar (0),
			'live_controller' :	StateVar (1),


			'focus' :						StateVar (None), 

			'part' :						StateVar (None), 

			'playing' : 				StateVar (0),
			'record' : 					StateVar (0),
			
			'tick' : 						StateVar (0),
		}

		keys = self.state.keys()
		for key in keys: 
			# initialize them in our __dict__
			setattr (self, key, self.state[key].value) 

		for key in keys: 
			# build the widget list
			self.state[key].widget = self.find_widget (key) or self.state[key]

		for key in keys: 
			# initialize the widget states
			self.state[key].widget.set_value (self.state[key].value)

	def widget (self, key):
		return self.state[key].widget

	def find_widget (self, name, widget = None):
		"""Finds a widget by name, climbs around the Gtk widget tree."""
		widget = widget or self.window
		if widget.get_name() == name: 
			return widget.get_data ('self') or widget
		try:
			for child in widget.children():
				found = self.find_widget (name, child)
				if found: 
					return found
		except:
			pass


	# Pygmy state change methods
	# usually called by the widgets
	
	def change_focus (self, focus):
		if self.focus == focus:
			return
		self.focus = focus
		
	def change_snap (self, value, widget = None):
		if not widget: 
			self.widget ('snap').set_value (value)
		self.snap = value
		
	# called by the partlist
	def change_part (self, part, widget = None):
		if not widget:
			self.widget ('part').set_value (part)
		self.part = part
		
		if not self.part:
			return
		
		if part.channel >= 0:
			self.change_channel (part.channel)
		if part.port >= 0:
			self.change_port (part.port)
			
	# playing, record
	def change_playing (self, value, widget = None):
		self.playing = \
		self.audible.playing = value
		if not widget:
			self.widget ('playing').set_value (self.playing)
		# display the exact position when we stop, please
		if not self.playing:
			self.widget ('tick').update_label()

	def change_record (self, value, widget = None):
		if not self.part or not self.song.index (self.part):
			self.record = 0
		else:
			self.record = not self.record
		if not widget:
			self.widget ('record').set_value (self.record)

	# tick
	def change_tick (self, value, widget = None):
		print "seek", value
		self.audible.seek (value)
		if not widget:
			self.widget ('tick').set_value (value)

	# Pygmy toggles
	def change_live_program (self, value, widget = None):
		self.live_program = value
		if not widget: self.widget ('live_program').set_value (value)
	
	def change_live_volume (self, value, widget = None):
		print value
		self.live_volume = value
		if not widget: self.widget ('live_volume').set_value (value)
	
	def change_live_pitch (self, value, widget = None):
		print value
		self.live_pitch = value
		if not widget: self.widget ('live_pitch').set_value (value)
	
	def change_live_controller (self, value, widget = None):
		self.live_controller = value
		if not widget: self.widget ('live_controller').set_value (value)
	
	# Pygmy parameters accessible through sliders
	def change_port (self, value, widget = None):
		if not widget: 
			self.widget('port').set_value (value)
		self.port = value
		self.widget('color').set_value (self.view.get_color (self.port, self.channel))

	def change_channel (self, value, widget = None):
		if not widget: 
			self.widget('channel').set_value (value)
		self.channel = value
		if len (self.io):
			self.io[0].write (SysEx ('\xf0\x00\x20\x33\x01\x10' + '\x72\x00' + chr(123) + chr (self.channel) + '\xf7'))
		self.widget('color').set_value (self.view.get_color (self.port, self.channel))

	def change_color (self, value, widget = None):
		if not widget: 
			self.widget('color').set_value (value)
		self.color = value
	
	def change_volume (self, value, widget = None):
		if not widget: 
			self.widget('volume').set_value (value)
		self.volume = value
		if self.live_volume:
			self.write_volume()

	def change_bank (self, value, widget = None):
		if not widget: 
			self.widget('bank').set_value (value)
		self.bank = value
		if self.live_program:
			self.write_program()

	def change_program (self, value, widget = None):
		if not widget: 
			self.widget('program').set_value (value)
		self.program = value
		if self.live_program:
			self.write_program()

	def change_note (self, value, widget = None):
		if not widget: 
			self.widget('note').set_value (value)
		self.note = value

	def change_velocity (self, value, widget = None):
		if not widget: 
			self.widget('velocity').set_value (value)
		self.velocity = value

	def change_velocity_off (self, value, widget = None):
		if not widget: 
			self.widget('velocity_off').set_value (value)
		self.velocity_off = value

	def change_pitch (self, value, widget = None):
		if not widget: 
			self.widget('pitch').set_value (value)
		self.pitch = value

	def change_controller (self, value, widget = None):
		if not widget: 
			self.widget('controller').set_value (value)
		self.controller = value

	def change_value (self, value, widget = None):
		if not widget: 
			self.widget('value').set_value (value)
		self.value = value
		if self.live_controller:
			self.write_controller()


	# Pygmy MIDI event functions

	def write_channel (self, *args):
		"""assigns the current port and channel to the current part"""
		if not self.part: return
		if not self.song.index (self.part): return
		self.part.port = self.port
		self.part.channel = self.channel
		self.widget ('part').update (self.part)
		if self.part in self.visible:
			self.view.redraw()

	def write_program (self, *args):
		self.write (Controller (0, self.bank))
		self.write (Program (self.program))

	def write_controller (self, *args):
		self.write (Controller (self.controller, self.value))

	def write_volume (self, *args):
		self.write (Controller (7, self.volume))

	def write_note (self, *args):
		self.write (Note (self.note, self.velocity))

	def write_note_off (self, *args):
		self.write (NoteOff (self.note, self.velocity_off))
		
	def write (self, event):
		"""stamp and write an event to output."""
		if not len (self.io):
			return
		event.port = self.port
		event.channel = self.channel
		print "WRITE:", event
		self.io.write (event)
		
	def sig_idle_after_pitch (self):
		idle_remove (self.idle_id)
		self.sig_periodic()

	def on_input (self, event):
		"""Simple midi-through"""
		#print "IN:", event
		if event.port != 1: return

		if event.type == PitchStatus:
			self.current_pitch = event.pitch
			if not self.pitch:
				# it is nice to have a direct response
				self.idle_id = idle_add (self.sig_idle_after_pitch) 
			if self.live_pitch:
				self.write (event)

		if event.type in (NoteStatus, NoteOffStatus):
			#event.note -= 12
			self.write (event)



###################################################################################

class TheLoop:
	"""Runs gtk_main_loop() in a separate thread.
		 Don't call into Gtk from the prompt!
		 Don't press ^C!
		 THIS IS BUGGY CODE if you hadn't already guessed so ... :)
	"""
	def __init__ (self):
		self.lock = thread.allocate_lock()
	
	def run (self):
		self.lock.acquire()
		mainloop()
		self.lock.release()
	
	def start (self):
		thread.start_new_thread (lambda args: args.run(), (self,))
		
	def stop (self):
		mainquit()
		self.lock.acquire()
		self.lock.release()
		

def mymainloop():
	while 1:
		main_iteration()

