/*
 *	hello all,
 *
 *	this file is just a playground for testing .cc code without
 *	firing up python.
 *
 */

#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>
#include <string.h>
#include <signal.h>

#include "midi-event.h"
#include "midi-part.h"
#include "midi-file.h"
#include "midi-song.h"
#include "midi-clock.h"
#include "midi-io.h"

#include "gtk-event-view.h"
#include "alsa-midi-io.h"

#include <stdexcept>

FILE * __debug_fp;

int _c_ref_sum = 0;

int do_something (int argc, char ** argv);

void 
sighandler (int signum)
{
	static int signalled = 0;
	
	fprintf (stderr, "signal %d caught ...\n", signum);
	if (++signalled == 1)
		gtk_main_quit();
	else
		exit (0);
}

int main (int argc, char ** argv)
{
	signal (SIGINT, sighandler);

	__debug_fp = fopen ("stderr", "a");
	if (!__debug_fp)
		__debug_fp = fopen ("/dev/null", "w");

	do_something (argc, argv);
	fprintf (stderr, "_c_ref_sum = %d\n", _c_ref_sum);
	return 0;
}

/*
 *	let's try the event view.
 *
 */
 
gint 
signal_destroy (GtkWidget * widget, void * data)
{
	gtk_main_quit();

	return true;
}

int do_something (int argc, char ** argv)
{
	//BREAKPOINT;
	alsa_port port;
	port.connect ("Emu");
	sleep (60);
	return 1;
}

int do_something_yo (int argc, char ** argv)
{
	gtk_init (&argc, &argv);

	gtk_event_view view;
	view.song.read ("files/bwv1068_air.mid");
	view.song.read ("files/adagio-3.mid");
	if (argc > 1)
		view.song.read (argv[1]);

	//view.song.read ("fullscale.mid");
	
	GtkWidget * window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_signal_connect (GTK_OBJECT (window), "destroy", GTK_SIGNAL_FUNC (signal_destroy), NULL);
	gtk_signal_connect (GTK_OBJECT (window), "key-press-event", GTK_SIGNAL_FUNC (gtk_event_view::key_event), &view);
	gtk_container_add (GTK_CONTAINER (window), view.widget);
	gtk_widget_show_all (window);
	gtk_widget_set_usize (window, gdk_screen_width() - 40, 600);
	
	gtk_main();
	return 0;
}

/*
 *	something else.
 *
 */

int do_something_2 (int argc, char ** argv)
{
	int i = 2;
	
	if (argc >= 3 && !sscanf (argv[2], "%d", &i))
	{
		fprintf (stderr, "%s is not a valid delay.\n", argv[2]);
		return 1;
	}

	__debug_fp = fopen ("stderr", "a");
	if (!__debug_fp)
		__debug_fp = fopen ("/dev/null", "w");

	m_song * song = new m_song();
	C_INCREF (song);

	if (argc >= 2 && !song->read (argv[1]))
	{
		fprintf (stderr, "cannot read %s\n - %s", argv[1], song->file.error);
		C_DECREF (song);
		return 1;
	}
	
	m_clock * clock = m_clock::get();

	if (!clock)
	{
		fprintf (stderr, "cannot make a rtc clock\n");
		C_DECREF (song);
		return 1;
	}
	
	C_INCREF (clock);
	
	m_io * io = new m_io (NULL);
	C_INCREF (io);
	
	clock->start();
	song->play();

	while (i--)
		sleep (1);
	
	C_DECREF (io);
	C_DECREF (clock);
	C_DECREF (song);

	return 0;
}
