/*
 *	midi-clock.h
 *
 *
 */

#ifndef _MIDI_CLOCK_
#define _MIDI_CLOCK_

#include <sys/poll.h>
#include <unistd.h>
#include <fcntl.h>

#include "arch-threads.h"
#include "midi-song.h"
#include "midi-io.h"

/*
 *	generic clock interface
 *
 */

class m_clock
:
public c_ref_obj,
public a_thread
{
	private:
		static m_clock *	the_clock;

	public:
											m_clock();

		static m_clock *	get();
		static void				release();

		virtual					 ~m_clock();
								 
		bool							running;

		c_list<m_connector *>		clients;	/* those that want to be pulsed */

		int								add_client (m_connector * _client);
		bool							remove_client (m_connector * _client);
		
		void							pulse();	/* to be called by derived classes */
											
		double						now();	/* returns (double) gettimeofday() */
		
		bool							set_resolution (int hz);
		bool							start();
		bool							stop();
	
	protected:
		struct {
			double	last;
			double	resolution;
			int			hz;
		} time;

		struct {
			int			fd[2];
			int			code;
		} cmd;

		a_mutex		mutex;

		enum {
			Quit		= 0,
			Start		= 1,
			Stop		= 2,
			Divide	= 3
		};

		virtual void * thread();

		int						do_command (int command);
};

#ifdef WITH_RTC

/*	
 *	m_rtc_clock 
 *
 *	clock based on /dev/rtc.
 */

class m_rtc_clock
:
public m_clock
{
	private:
		a_mutex 			mutex;

		struct {
			int			fd;
			int			hz;
		} rtc;
		
	public:
									m_rtc_clock();
		virtual			 ~m_rtc_clock();
						 
	private:
		virtual void * thread();

		int						thread_do_command (int command);
};

#endif  /* WITH_RTC */

#endif  /* _MIDI_CLOCK_ */
