/*
 *	midi-event-cursor.cc
 *
 */
 
#include <assert.h>

#include "midi-part.h"
#include "midi-song.h"

/*
 *	m_event_cursor.
 *
 */
 
m_event_cursor::m_event_cursor (m_part * _part, int i)
:
c_ref_obj ("m_event_cursor")
{
	C_INCREF (_part);
	init (_part, i);
}

m_event_cursor::~m_event_cursor()
{
	C_DECREF (part);
}

void
m_event_cursor::init (m_part * _part, int i)
{
	if (!_part)
		assert (part && "init (NULL) without a previous part");
	else
		part = _part;
	
	index = i;
	if (i >= part->events.count)
	{
		event = NULL;
		tick = 0;
	}
	else
	{
		event = part->event(index);
		tick = event->get_tick();
	}
}

/*
 *	move_to.
 *	
 */

m_event *
m_event_cursor::move_to (m_tick t)
{
	tick = t;
	
	if (!part->events.count)
	{
		index = -1;
		event = NULL;
	}
	else
	{
		index = part->index_of_tick (tick);

		if (index >= part->events.count)
		{
			index = part->events.count;
			event = NULL;
		}
		else
		{
			assert (index >= 0);
			event = part->event (index);
			tick = event->get_tick();
		}
	}
	
	return event;
}

/*
 *	step_*
 *	
 */

m_event *
m_event_cursor::step_fwd()
{
	if (event == NULL)
		return NULL;
		
	index++;
	
	if (index >= part->events.count)
	{
		event = NULL;
	}
	else
	{
		event = part->event (index);
		tick = event->get_tick();
	}

	return event;
}

/*
 *	
 *	
 */

m_event * 
m_event_cursor::step_rev()
{
	if (event == NULL)
		return NULL;
		
	index--;
	
	if (index < 0)
	{
		index = 0;
		event = NULL;
		tick = 0;
	}
	else
	{
		event = part->event (index);
		tick = event->get_tick();
	}

	return event;
}


