/*
 * midi-file.h
 *
 * a collection of midi parts along with the possibility of
 * reading and writing a standard midi file
 *
 */

#ifndef _MIDI_FILE_
#define _MIDI_FILE_
#include <stdio.h>
#include <malloc.h>
#include "arch-types.h"
#include "common-list.h"
#include "common-refcount.h"
#include "midi-meta-types.h"
#include "midi-event.h"
#include "midi-part.h"

typedef struct 
{
	char	MThd[4];
	dword	length;
	word	format;
	word	tracks;
	word	division;
} m_fileheader;


class m_file 
: public c_ref_obj
{
	public:
		struct {
			FILE *		fp;
			long			remaining;
			char 			error[120];
			m_tick		tick;
			int				debug;
		} file;
		
		char *				name;
		word					division;

		c_ref_list		parts;
		m_part *			part (int i) {return (m_part *) parts[i];}
		m_part &			operator[] (int i) {return *part(i);}

		
									m_file(); 
									m_file (char * _ref_name); 
									
		virtual			 ~m_file();
					
		virtual int		add_part (m_part * part);
		virtual bool	remove_part (m_part * part);

		virtual int		read (char *);
		int						write (char * = 0);
		virtual void	clear();

		void					set_name (char *);
		char *				get_error() {return file.error;}

		void					repr (c_char_buffer *, char limiter);

	private:
		void					reset_error();

		/* read helpers */
		bool					read_event (m_part *);
		
		long					read_varlen();
		dword					read_dword();
		word					read_word();
		byte					read_byte();
		
		/* write helpers */
		bool					write_event (m_active_notes * playing, m_event * ev);
		bool					write_note_offs (m_active_notes * playing, m_tick next);

		bool					write_data (void * pdata, long length);
		bool					write_varlen (unsigned long value);
		bool					write_dword (dword value);
		bool					write_word (word value);
		bool					write_byte (byte value);
};

#endif
