/*
 *	midi-meta-events.h
 *	
 */

#ifndef _MIDI_META_EVENTS_
#define _MIDI_META_EVENTS_

#define M_META_SEQ_NO	0x00
/*
FF 00 02                     Sequence Number
    This  optional  event, which must occur at the beginning  of  a  track,
    before  any  nonzero  delta-times, and before  any  transmittable  MIDI
    events, specifies the number of a sequence. In a format 2 MIDI File, it
    is used to identify each "pattern" so that a "song" sequence using  the
    Cue  message to refer to the patterns. If the ID numbers  are  omitted,
    the sequences' lacations in order in the file are used as defaults.  In
    a format 0 or 1 MIDI File, which only contain one sequence, this number
    should  be  contained  in the first (or only)  track.  If  transfer  of
    several multitrack sequences is required, this must be done as a  group
    of format 1 files, each with a different sequence number.
*/
#define M_META_TEXT 0x01
/*
FF 01 len text               Text Event
    Any amount of text describing anything. It is a good idea to put a text
    event right at the beginning of a track, with the name of the track,  a
    description  of its intended orchestration, and any  other  information
    which the user wants to put there. Text events may also occur at  other
    times in a track, to be used as lyrics, or descriptions of cue  points.
    The text in this event should be printable ASCII characters for maximum
    interchange.  However, other characters codes using the high-order  bit
    may be used for interchange of files between different programs on  the
    same  computer which supports an extended character set. Programs on  a
    computer  which  does not support non-ASCII  characters  should  ignore
    those characters.

    Meta-event  types 01 through 0F are reserved for various types of  text
    events,  each of which meets the specification of text  events  (above)
    but is used for a different purpose:
*/
#define M_META_COPY	0x02
/*
FF 02 len text               Copyright Notice
    Contains a copyright notice as printable ASCII text. The notice  should
    contain the characters (C), the year of the copyright, and the owner of
    the  copyright. If several pieces of music are in the same  MIDI  File,
    all of the copyright notices should be placed together in this event so
    that it will be at the beginning of the file. This event should be  the
    first event in the track chunk, at time 0.
*/
#define M_META_SEQ_NAME	0x03
/*
FF 03 len text               Sequence/Track Name
    If in a format 0 track, or the first track in a format 1 file, the name
    of the sequence. Otherwise, the name of the track.
*/
#define M_META_INSTRUMENT 0x04
/*
FF 04 len text               Instrument Name
    A description of the type of instrumentation to be used in that  track.
    May  be  used  with the MIDI Prefix meta-event to  specify  which  MIDI
    channel the description applies to, or the channel may be specified  as
    text in the event itself.
*/
#define M_META_LYRIC 0x05
/*
FF 05 len text               Lyric
    A  lyric to be sung. Generally, each syllable will be a seperate  lyric
    event which begins at the event's time.
*/
#define M_META_MARKER 0x06
/*
FF 06 len text               Marker
    Normally  in a format 0 track, or the first track in a format  1  file.
    The  name of that point in the sequence, such as a rehersal  letter  or
    section name ("First Verse", etc.)
*/
#define M_META_CUE 0x07
/*
FF 07 len text               Cue Point
    A description of something happening on a film or video screen or stage
    at that point in the musical score ("Car crashes into  house",
    "curtain opens", "she slaps his face", etc.)
*/
#define M_META_CHANNEL 0x20
/*
FF 20 01 cc                  MIDI Channel Prefix
    The  MIDI  channel  (0-15)  containted in this event  may  be  used  to
    associate a MIDI channel with all events which follow, including System
    exclusive  and meta-events. This channel is "effective" until the  next
    normal  MIDI event (which contains a channel) or the next MIDI  Channel
    Prefix meta-event. If MIDI channels refer to "tracks", this message may
    into  a  format 0 file, keeping their non-MIDI data associated  with  a
    track. This capability is also present in Yamaha's ESEQ file format.
*/
#define M_META_PORT 0x21
/*
FF 21 01 pp                  MIDI Port
		<added Oct 13 2000>
		Port prefix.
*/
#define M_META_EOT	0x2F
/*
FF 2F 00                     End of Track
    This  event  is not optional. It is included so that  an  exact  ending
    point may be specified for the track, so that an exect length, which is
    necessary for tracks which are looped or concatenated.
*/
#define M_META_TEMPO	0x51
/*
FF 51 03 tttttt              Set Tempo
                             (in microseconds per MIDI quarter-note)
    This   event  indicates  a  tempo  change.  Another  way   of   putting
    "microseconds  per  quarter-note" is "24ths of a microsecond  per  MIDI
    clock".  Repersenting tempos as time per beat instead of beat per  time
    allows absolutly exact long-term synchronization with a time-based sync
    protocol  such  as SMPTE time code or MIDI time code.  This  amount  of
    accuracy  provided by this tempo resolution allows a four-minute  piece
    at  120 beats per minute to be accurate within 500 usec at the  end  of
    the  piece. Ideally, these events should only occur where  MIDI  clocks
    would  be  located -- this convention is intended to  guarntee,  or  at
    least   increase   the   liklihood,   of   compatibility   with   other
    synchronization  devices so that a time signature/tempo map  stored  in
    this format may easily be transfered to another device.
*/
#define M_META_SMPTE	0x54
/*
FF 54 05 hr mn se fr ff      SMPTE Offset
    This  event, if present, designates the SMPTE time at which  the  track
    chunk  is supposed to start. It should be present at the  beginning  of
    the  track,  that is, before any nonzero delta-times,  and  before  any
    transmittable  MIDI  events. the hour must be encoded  with  the  SMPTE
    format, just as it is in MIDI Time Code. In a format 1 file, the  SMPTE
    Offset must be stored with the tempo map, and has no meaning in any  of
    the other tracks. The ff field contains fractional frames, in 100ths of
    a  frame,  even in SMPTE-based tracks which specify a  different  frame
    subdivision for delta-times.
*/
#define M_META_TIME 0x58
/*
FF 58 04 nn dd cc bb         Time Signature
    The  time signature is expressed as four numbers. nn and  dd  represent
    the  numerator  and denominator of the time signature as  it  would  be
    notated.  The  denominator is a neqative power of two: 2  represents  a
    quarter-note,  3  represents  an eighth-note,  etc.  The  cc  parameter
    expresses  the  number  of MIDI clocks in a  metronome  click.  The  bb
    parameter  expresses  the  number  of  notated  32nd-notes  in  a  MIDI
    quarter-note (24 MIDI clocks). This was added because there are already
    multiple  programs which allow a user to specify that what MIDI  thinks
    of as a quarter-note (24 clocks) is to be notated as, or related to  in
    terms of, something else.

    Therefore, the complete event for 6/8 time, where the metronome  clicks
    every three eighth-notes, but there are 24 clocks per quarter-note,  72
    to the bar, would be (in hex):

               FF 58 04 06 03 24 08

    That is, 6/8 time (8 is 2 to the 3rd power, so this is 06 03), 36  MIDI
    clocks  per dotted-quarter (24 hex!), and eight notated 32nd-notes  per
    quarter-note.
*/
#define M_META_KEY	0x59
/*
FF 59 02 sf mi               Key Signature
    sf = -7: 7 flats
    sf = -1: 1 flat
    sf =  0: key of C
    sf =  1: 1 sharp
    sf =  7: 7 sharps

    mi =  0: major key
    mi =  1: minor key
*/
#define M_META_META	0x7F
/*
FF 7F len data               Sequencer Specific Meta-Event
    Special requirements for particular sequencers may use this event type:
    the  first  byte or bytes of data is a manufacturer ID (these  are  one
    byte,  or  if the first byte is 00, three bytes). As with  MIDI  System
    Exclusive,  manufacturers  who define something using  this  meta-event
    should  publish  it  so that others may be used by  a  sequencer  which
    elects  to  use  this as its only file format;  sequencers  with  their
    established  feature-specific  formats  should probably  stick  to  the
    standard features when using this format.
*/

#endif
