/*	
 *	midi-part.h
 *	
 */

#ifndef _MIDI_PART_
#define _MIDI_PART_

#include "midi-event.h"
#include "midi-connector.h"

class m_event_cursor;

/*
 *	m_event_cursor
 * 
 *	keeps track of events inside a part even if other events are 
 *	inserted or removed at a lower index.
 *
 */

class m_part;

class m_event_cursor
:
public c_ref_obj
{
	public:
		m_part *		part;
		
	
		m_event	*		event;
		m_tick			tick;
		int					index;
		
								m_event_cursor (m_part * _part, int i = 0);
							 ~m_event_cursor();
							 
		void				init (m_part * _part = 0, int i = 0);
	
		m_event *		move_to (m_tick t);

		m_event *		step_fwd();
		m_event *		step_rev();
};

/*
 *	m_part
 *
 *	is a list of events. 
 *	the member functions take care of keeping the events
 *	sorted by event.generic.tick
 *
 */

class m_part
: 
public m_connector
{ 
	public:
		m_tick			division;
		int					channel;
		int					port;

		c_ref_list	events;
		m_event *		event (int i) {return (m_event *) events[i];}
		m_event &		operator[] (int i) {return *event(i);}

		struct {
			m_tick			tick;
			int					index;
		} last_add;

		int					index_of_tick	(m_tick t);

		c_ref_list	cursors;
		m_event_cursor *	cursor (int i) {return (m_event_cursor *) cursors[i];}
		
	public:
								m_part();
								m_part (m_part * other, int slice_start, int slice_end);
		virtual		 ~m_part();

		void				initialize (m_tick _division);
		void				copy (m_part *);
		void				copy_slice (m_part *, int start, int index);
		void				remove_slice (int start, int end);
		void				clear();
		
	public:
		m_event_cursor *	new_cursor (int index = 0);
		bool							del_cursor (m_event_cursor *);

	/* m_connector:: */
	public:
		virtual bool			event_in (m_event * e, m_event_stamp * = 0) {return add_event (e) >= 0;}
		
	public:
		int					add_event (m_event * e);
		m_event *		parse_note_off (m_event * e, m_tick at = -1);
		
		bool				move_event (int index, m_tick to);

		bool				remove (int i);
		bool				remove_event (m_event * e);
		
	public:
		void				repr (c_char_buffer *, char limiter);

	public:
		void				sort() 		{qsort (0, events.count - 1);}
		void				qsort (int lo, int hi);

	private:
		void				swap_events (int i1, int i2);

	public:
		m_event *		find_note_on (m_tick t, byte channel, byte note);
		m_event *		find_non_meta (m_tick t = 0, int direction = +1);
		m_event *		find_meta (byte type, m_tick t = 0, int direction = +1);
		m_event *		find_event (byte type, int index = 0, int direction = +1);
		
	public:
		int					get_port();
		int					get_channel();
		void				set_port (int port);
		void				set_channel (int channel);

	public:
		a_rwlock		lock;
					
		m_part *		lock_read() {lock.lock_read(); return this;}
		m_part *		unlock_read() {lock.unlock_read(); return this;}

		m_part *		lock_write() {lock.lock_write(); return this;}
		m_part *		unlock_write() {lock.unlock_write(); return this;}
};

/*
 *	m_active_notes
 *
 *	inserts only notes, 
 *	helpful for a simple list of currently sounding notes
 *
 */

class m_active_notes 
: 
public m_part
{
	public:
									m_active_notes();
				
	virtual bool		event_in (m_event * e, m_event_stamp * = 0);
	m_event *				next_before (m_tick tick);
};
	
#endif
