/*
 *	midi-song.h
 *
 *	the m_song is driven by m_sync and outputs
 *	to a m_connector.
 *	
 */

#ifndef _MIDI_SONG_
#define _MIDI_SONG_

#include "midi-part.h"
#include "midi-file.h"
#include "midi-io.h"

class m_song;

/*
 *	beat counter
 *
 */

class m_beat
{
	public:
		int			bar;
		int			beat;
		int			fraction;

		int			numerator, denominator;
		m_tick	division;

		struct {
			m_tick	current;
			m_tick	of_last_event;
			m_tick	per_beat;
			m_tick	quantize;
		} tick;
		
		m_event		click;
		m_event		clock;


							m_beat(); 
							m_beat (m_beat &other); 

		void			reset (int _division);
		void			parse_time (m_time_event *, m_connector * = 0);
		
		m_tick		next_beat (m_tick click, m_tick _quantize);
		void			advance (m_tick to, m_connector * = 0);
		
		void			advance_to_tick (m_tick _tick);
		void			seek (int _bar, int _beat, int _fraction);
};

/* aux structure to remember playing notes */
typedef struct 
{
	m_tick	tick;
	int			note;
	int			channel;
	int			port;
	int			velocity;
} PlayingNote;

/*
 *	m_song.
 *
 *	the songpointer.
 *
 */

class m_song 
: 
public m_file
{
	public:
		a_mutex		mutex;
		c_list<m_event_cursor *> cursors;
		m_active_notes notes;

		bool			playing;

		m_beat		beat;
		m_event		clock;
		m_event		default_tempo;
		m_event		default_time;

		/* _virtual tick and time are advanced in stopped mode */
		struct {
			m_tick	current;
			m_tick	next;
			m_tick	of_last_tempo;
			m_tick	per_clock;
		} tick;

		struct {
			double	current; 
			double	per_quarter;
			double	per_tick;

			double	of_last_tempo;
			
			int			usec_per_quarter;
		} time;
		
		struct {
			m_tick	start;
			m_tick	end;
			int			count;
		} loop;

		
									m_song();
		virtual				~m_song();

		void					reset();

		bool					set_file (m_file *);
		virtual int		read (char * filename);
		virtual int		add_part (m_part *);
		virtual bool	remove_part (m_part *);
		virtual void	clear();

		void					set_division (int _division);

		bool					play();
		bool					stop();

		m_tick				get_tick() {return tick.current;}
		double				get_time() {return time.current;}

		void					advance (double dt, m_connector *);
		void					advance_to_time (double _time, m_connector *);
		void					advance_to_tick (m_tick _tick, m_connector * = 0);
		
									/* seek min (_time, _tick) */
		bool					seek (double _time, m_tick _tick = 0, m_connector * = 0);
		bool					seek (int _bar, int _beat, m_tick _fraction, m_connector * = 0);

		void					stop_active_notes (m_tick until = M_TICK_MAX, m_connector * = 0);

	private:
		void					emit_event (m_event_cursor *, m_connector * = 0);
		bool					parse (m_event *, m_connector * out = 0);
		void					parse_tempo (m_tempo_event *);

	public:
		double				tick2time (m_tick _tick);
		m_tick				time2tick (double _time);
};

#endif /* _MIDI_SONG_ */
