/*
 *	midi-xy-mapper.cc
 *
 *	map events to xy coordinates.
 *
 */

#include <assert.h>

#include "midi-xy-mapper.h"

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *	m_x_mapper
 *	
 */

m_x_mapper::m_x_mapper (m_song * _song)
{
	song = _song;
	
	magnification = 1;
	x0 = 0;
	
	set_magnification (2, 0);
}

m_x_mapper::~m_x_mapper()
{
}

m_tick
m_x_mapper::x2tick (int x)
{
	return (m_tick) ((double) (x - x0) * ticks_per_pixel);
}

int
m_x_mapper::tick2x (m_tick tic)
{
  return x0 + (int) ((double) tic / ticks_per_pixel);
}

/*
 *	scroll
 *
 */

void
m_x_mapper::scroll (int dx)
{
	x0 += dx;
}

void
m_x_mapper::set_magnification (int _magnification, int x)
{
	m_tick tick = x2tick (x);

	magnification = _magnification - 4;

	if (magnification < 0)
		pixels_per_tick = 1.0 / (double) (1 << -magnification);
	else
		pixels_per_tick = 1.0 * (double) (1 << magnification);
	
	ticks_per_pixel = 1.0 / pixels_per_tick;

	scroll (x - tick2x (tick));
}

/* * * * * * * * * * * * * * * *
 *
 *	m_y_mapper
 *	
 */

m_y_mapper::m_y_mapper()
:
y2n()
{
	magnification = 1;
	y0 = 0;
	
	init();
	
	set_magnification (2, 0);
}

m_y_mapper::~m_y_mapper()
{
}

/*
 *
 *	
 */

void
m_y_mapper::init()
{
	memset (is_staff, 0, sizeof (is_staff));
	memset (gravity, 0, sizeof (gravity));
	memset (alpha, 0, sizeof (gravity));

	int zone_index = 0;
	int	current_y = 0;

	for (int note = 0; note < 128; note++)
	{
		switch ((note + 12) % 24)
		{
			case 0:
			case 7:
			case 14:		/* 'D' actually is a 3 fold zone */
			case 17:
				is_staff[note] = true;
				zone_index = 0;
				break;
			case 4:
			case 11:
			case 21:
				is_staff[note] = true;
				zone_index = 1;
				break;
		}

		if (is_staff[note])
			current_y += 4,
			n2y[note] = current_y;
			
		else
			n2y[note] = current_y + zone_index;
			
		zone_index++;

		if (note < 43)
			gravity [note] = +1;
			
		else if (note <= 57)
			;
			
		else if (note <= 60)
			gravity [note] = +1;
			
		else if (note < 63)
			gravity [note] = -1;

		else if (note <= 77)
			;

		else
			gravity [note] = -1;

		if (is_staff[note])
		{
			if (!gravity [note])
				alpha [note] = 0xff;
			else 
			switch (note % 12)
			{
				case 0:
					alpha[note] = 0x60; break;
				case 5:
				case 7:
					#if 0
					alpha[note] = 0x30; break;
					#endif
				default:
					alpha[note] = 0x00; break;
			}
		 }
	}

	y2n.clear();
	
	int y = 0;
	for (int note = 0; note < 127; note++)
	{
		_dbgprintf (0, "y=%d n2y[% 3d]=% 3d next %d\n", y, note, n2y[note], n2y[note+1]);
		while (y < n2y [note + 1])
		{
			y2n.insert_item (note);
			y++;
		}
	}

	#if 0
	for (int note = 0; note < 128; note++)
		_dbgprintf (1, "n=%.3d y=%.3d [%.3d] %c\n", note, n2y[note], note2y (note), ' ' + (int) is_staff[note]);
	for (y = 0; y < y2n.count; y++)
		_dbgprintf (1, "y2n[% 3d]=% 3d\n", y, y2n[y]);
	#endif
}

void
m_y_mapper::scroll (int dy)
{
	y0 += dy;
}

void
m_y_mapper::set_magnification (int _magnification, int y)
{
	int note = y2note (y);

	magnification = _magnification;
	
	if (magnification < 0)
		pixels_per_note = 1.0 / (double) (1 << -magnification);
	else
		pixels_per_note = 1.0 * (double) (1 << magnification);
	
	notes_per_pixel = 1.0 / pixels_per_note;

	y0 = 300 + (int) (pixels_per_note * 72);

	scroll (y - note2y (note));
}

void
m_y_mapper::set_position (int note, int y)
{
	scroll (y - note2y (note));
}

int
m_y_mapper::note_height()
{	
	return (int) (3 * pixels_per_note) | 1;
}

int
m_y_mapper::y2note (int y)
{
	if (!y2n.count)
		return 0;

	y = (int) (((double) (y0 - y)) / pixels_per_note);
	
	if (y < y2n[0])
		return -1;
	
	if (y > y2n[y2n.count - 1])
		return 128;
	
	return y2n[y];
}

int
m_y_mapper::note2y (int note)
{
	if (note < 0)
		note = 0;
	if (note >= 128)
		note = 127;
  return y0 - (int) (pixels_per_note * n2y [note]);
}





