from _midithing import *
import os

# more Port() openers

def OSSPort (port = 0):
	"""Opens an OSS MIDI port and returns a Port object."""
	return Port ('/dev/midi%d' % (port))

def RawALSAPort (card, port = 0):
	"""Opens an ALSA MIDI port in raw midi mode and returns a Port object."""
	return Port ('/dev/snd/midiC%dD%d' % (card, port))

# transposers

def longer (thing, amount):
	"""Modifies note lengths."""
	if type (thing) != EventType:
		for thing in thing:
			longer (thing, amount)
		return
	if thing.type == NoteStatus:
		try: thing.length += amount
		except ValueError:
			if amount < 0: thing.length = 1

def louder (thing, amount):
	"""Modifies velocity data."""
	if type (thing) != EventType:
		for thing in thing:
			louder (thing, amount)
		return
	if thing.type in (NoteStatus, NoteOffStatus):
		try: thing.velocity += amount
		except ValueError:
			if amount < 0: thing.velocity = 1
			if amount > 0: thing.velocity = 127

def transpose (thing, semis):
	"""Transposes a thing by semitones."""
	if type (thing) != EventType:
		for thing in thing:
			transpose (thing, semis)
		return
	if thing.type in (NoteStatus, NoteOffStatus):
		try: 
			thing.note += semis
		except ValueError:
			if semis < 0: thing.note = 0
			if semis > 0: thing.note = 127

def set_channel (thing, channel):
	"""Sets the channel on a thing."""
	if type (thing) != EventType:
		for thing in thing:
			set_channel (thing, channel)
		return
	try:
		thing.channel = channel
	except AttributeError:
		pass

# Scale class 

class Scale:
	major = [0,2,4,5,7,9,11]
	minor = [0,2,3,5,7,8,10]
	
	def __init__ (self, base = 0, steps = major):
		self.base = base
		self.steps = steps

	def at_step (self, step):
		step %= len (self.steps)
		
		steps = self.steps[step:]
		for s in self.steps[:step]:
			steps += [s + 12]

		return Scale (base = self.steps[step], steps = steps)
			
	def arpeggio (self, note, steps = 4, interval = 0, length = 48):
		part = Part()
		for n in (0,2,4,6,8,10,12,14)[:steps]:
			e = Note (note, length=length)
			e.note += self.steps [n] - self.base
			part.add (e, tick = interval * n / 2)
		return part

	def transpose (self, semis):
		steps = []
		for s in self.steps:
			steps += [s + semis]
		self.steps = steps
		self.base += semis

	def __repr__ (self):
		return "Scale (base = %d, steps = %s)" % (self.base, self.steps)

