"""Miscellaneous widgets for the Flying Circus"""
from midithing import *
from gtk import *
from inputs import *
import GTK
import pixmaps

class PartList (GtkTable, Input):
	column_width = 24
	row_height = 18
	audible_xpm = None
	visible_xpm = None
	
	def make_pixmaps (self):
		if self.audible_xpm:
			return
		w = self.pygmy.window

		self.audible_xpm = (
			pixmaps.XPM (w, 'audible-off'), 
			pixmaps.XPM (w, 'audible')
		)

		self.visible_xpm = ( 
			pixmaps.XPM (w, 'visible-off'),
			pixmaps.XPM (w, 'visible')
		)

		return
	
	def make_list (self):
		titles = ("name", "  ", "aud", "vis")
		list = GtkCList (4, titles)
		vscroll = GtkVScrollbar()

		list.set_row_height (self.row_height)
		list.set_selection_mode (GTK.SELECTION_BROWSE)
		#self.list.column_titles_hide()

		for i in range (3):
			list.set_column_justification (i + 1, GTK.JUSTIFY_CENTER)		
			list.set_column_min_width (i + 1, self.column_width)		
			list.set_column_max_width (i + 1, self.column_width)
			
		list.connect ("select-row", self.sig_select)
		
		list.set_vadjustment (vscroll.get_adjustment())
		
		return (list, vscroll)

	#   __init__
	def __init__ (self, pygmy, name):
		"""_init_"""
		
		GtkTable.__init__ (self, rows=2, cols=2)
		Input.__init__ (self, pygmy, name)
		
		self.part = None
		self.make_pixmaps()

		# build
		(self.list, vscroll) = self.make_list()

		self.attach (vscroll, 1, 2, 0, 1, SHRINK, EXPAND|FILL)
		
		self.attach (self.list, 0, 1, 0, 1, EXPAND|FILL|SHRINK, FILL)

		self.total_width = 240
		self.list.set_column_width (0, self.total_width - 4 * 4 - 3 * self.column_width)		


	def change (self, part):
		"""called when a GdkEvent led to a state change"""
		self.value = part
		Input.change (self, part)


	def set_part (self, part):
		self.set_value (part)

	def set_value (self, part):
		"""called when Pygmy is changing the state"""
		self.value = part
		self.part = part
		row = self.list.find_row_from_data (part)
		self.list.select_row (row, 0)


	def update_label (self):
		"""called when changing state"""
		pass


	def set_song (self, song):
		"""called by main at song change"""
		list = self.list
		
		list.freeze()
		list.clear()
		
		for part in song:
			self.add_part (part)

		self.list.thaw()
	
	def add_part (self, part):
		row = self.list.append ((part.name, ' ',' ',' '))
		self.list.set_row_data (row, part)
		self.update (part)
		self.list.select_row (row, 0)

	def remove_part (self, part):
		row = self.list.find_row_from_data (part)
		self.list.freeze()
		self.list.remove (row)
		if row == len (self.pygmy.song): row -= 1
		self.list.select_row (row, 0)
		self.list.thaw()

	def update (self, part):
		"""[re]initializes a row's pixmaps"""
		list = self.list
		row = list.find_row_from_data (part)
		list.set_text (row, 0, part.name)
		if row:
			list.set_pixmap (row, 2, self.audible_xpm[1])
			list.set_pixmap (row, 3, self.visible_xpm[1])

		if part.port >= 0 and part.channel >= 0:
			# the instrument color
			color = self.pygmy.view.get_color (part.port, part.channel)
			
			pixmap = pixmaps.ColoredPixmap (
				self.pygmy.window, 
				color, 
				self.column_width, 
				12
			)
			self.list.set_pixmap (row, 1, pixmap)
		
		
	def sig_select (self, list, row, column, event):
		part = self.list.get_row_data (row)
		if row:
			# toggle audible
			if column == 2:
				audible = self.pygmy.toggle_audible (part)
				self.list.set_pixmap (row, 2, self.audible_xpm [audible])
			# toggle visible
			elif column == 3:
				visible = self.pygmy.toggle_visible (part)
				self.list.set_pixmap (row, 3, self.visible_xpm [visible])
				self.pygmy.view.redraw()
		if part == self.part: return
		self.change (part)
			

