/*
 *	python-callback.cc
 *
 */

#include "python-types.h"

#define	DBG_EVENTS	1
#define	DBG_RETURN	1

py_callback::py_callback()
: 
m_connector ("py_callback")
{
	/* make a new thread state for callbacks from m_clock.
	 * it is assumed this is always called from the main thread 
	 * so it is not necessary to acquire the python lock here.
	 * same below.
	 */

	PyInterpreterState * interp = PyThreadState_Get()->interp;
	tstate = PyThreadState_New (interp);

	callback = NULL;
}

py_callback::~py_callback()
{
	/* see above. */
	
	PyThreadState_Clear (tstate);
	PyThreadState_Delete (tstate);

	Py_XDECREF (callback);
}

/*
 *	set the associated Python callable
 *
 */
 
bool
py_callback::set_callback (PyObject * _callback)
{
	if (!_callback)
	{
		PyErr_SetString (PyExc_TypeError, "give me a callable.");
		return false;
	}
	
	if (!PyCallable_Check (_callback))
	{
		PyErr_SetString (PyExc_TypeError, "must be callable.");
		return false;
	}
	
	Py_INCREF (_callback);
	
	callback = _callback;

	return true;
}

/*
 *	callback with a tuple
 *
 */

bool
py_callback::call_with_tuple (PyObject * args)
{
	PyThreadState * h_tstate = NULL;

	if (tstate)
	{
		PyEval_AcquireLock();
		h_tstate = PyThreadState_Swap (tstate);
	}
	
	PyObject * r = PyEval_CallObject (callback, args);

	if (PyErr_Occurred())
	{
		PyErr_Print();
	}
	
	#if defined (DEBUG) && DBG_RETURN > 0
	if (r)
	{
		fputs ("[callback return] ", __debug_fp);
		PyObject_Print (r, __debug_fp, 0);
		fputs ("\n", __debug_fp);
	}
	#endif

	if (tstate)
	{
		PyThreadState_Swap (h_tstate);
		PyEval_ReleaseLock();
	}

	Py_XDECREF (r);

	return r != NULL;
}

/*
 *	callback with an event
 *
 */

bool
py_callback::event_in (m_event * e, m_event_stamp * stamp)
{
	if (!callback)
		return false;

	PyThreadState * h_tstate = NULL;

	if (tstate)
	{
		PyEval_AcquireLock();
		h_tstate = PyThreadState_Swap (tstate);
	}
	
	bool result = false;
	
	if (stamp)
	{
		e = new m_event (e); /* we can do that because it will be deleted by C_DECREF */
		stamp->apply (e);
	}

	C_INCREF (e);

	PyObject * args = PyTuple_New (1);
	if (args)
	{
		PyTuple_SetItem (args, 0, (PyObject *) m_event_new_from_event (e));
		
		PyObject * r = PyEval_CallObject (callback, args);

		Py_XDECREF (r);
		Py_DECREF (args);
	}

	C_DECREF (e);

	if (tstate)
	{
		PyThreadState_Swap (h_tstate);
		PyEval_ReleaseLock();
	}

	return result;
}
