/*
 *	python-midi-song.cc
 *
 */

#include "python-types.h"

/*
 *	new m_song
 *
 */

m_song_object *
m_song_new()
{
	return m_song_new_from_song (NULL);
}

m_song_object *
m_song_new_from_song (m_song * song)
{
	m_song_object * self = PyObject_NEW (m_song_object, &m_song_py_type);
	if (!self)
		return NULL;

	if (!(self->song = song))
	{
		self->song = new m_song();
		if (!self->song)
		{
			PyMem_DEL (self);
			PyErr_NoMemory();
			return NULL;
		}
	}
		
	_dbgprintf (DBG_REF, "<m_song_object %x>\n", (int) self);

	C_INCREF (self->song);

	return self;
}

/*
 *	delete m_song
 *
 */

static void
m_song_del (m_song_object * self)
{
	_dbgprintf (DBG_REF, "<~m_song_object %x>\n", (int) self);

	C_DECREF (self->song);

	PyMem_DEL (self);
}

/*
 *	m_song.read
 *
 */

static PyObject *
m_song_read (m_song_object * self, PyObject * args)
{
	char * name = NULL;

	if (!PyArg_ParseTuple (args, "s;filename", &name))
		return NULL;
	
	if (!self->song->read (name))
	{
		PyErr_SetString (PyExc_IOError, self->song->get_error());
		return NULL;
	}
	
	Py_INCREF (Py_None);
	return Py_None;
}

/*
 *	m_song.write
 *
 */

static PyObject *
m_song_write (m_song_object * self, PyObject * args)
{
	char * name = NULL;

	if (!PyArg_ParseTuple (args, "|s;optional filename", &name))
		return NULL;
	
	self->song->write (name);
	char * err = self->song->get_error();
	
	if (*err)
	{
		PyErr_SetString (PyExc_IOError, err);
		return NULL;
	}
	
	return Py_BuildValue ("s", self->song->name);
}

/*
 *	m_song.add (m_part *)
 *
 */

static PyObject *
m_song_add (m_song_object * self, PyObject * args)
{
	m_part_object * object;
	if (!PyArg_ParseTuple 
		(
			args,
			"O!;needs an "M_PART_NAME, 
			&m_part_py_type, &object
		)
	) return NULL;

	self->song->add_part (object->part);

	Py_INCREF (Py_None); 
	return Py_None;
}

/*
 *	m_song.index (m_part *)
 *
 */

static PyObject *
m_song_index (m_song_object * self, PyObject * args)
{
	m_part_object * object;
	if (!PyArg_ParseTuple 
		(
			args,
			"O!;"M_PART_NAME, 
			&m_part_py_type, &object
		)
	) return NULL;

	int i = self->song->parts.find (object->part);

	if (i < 0)
	{
		PyErr_SetString (PyExc_IndexError, "this "M_PART_NAME" is not in this "M_SONG_NAME".");
		return NULL;
	}

	return PyInt_FromLong (i);
}

/*
 *	m_song.clear()
 *
 */

static PyObject *
m_song_clear (m_song_object * self, PyObject * args)
{
	if (!PyArg_ParseTuple (args, ""))
		return NULL;

	self->song->clear();
	
	Py_INCREF (Py_None); 
	return Py_None;
}

/*
 *	m_song.remove (m_part *)
 *
 */

static PyObject *
m_song_remove (m_song_object * self, PyObject * args)
{
	m_part_object * object;
	if (!PyArg_ParseTuple 
		(
			args,
			"O!;"M_PART_NAME, 
			&m_part_py_type, &object
		)
	) return NULL;

	if (!self->song->remove_part (object->part))
	{
		PyErr_SetString (PyExc_AssertionError, "this "M_PART_NAME" is not in the "M_SONG_NAME".");
		return NULL;
	}

	Py_INCREF (Py_None); 
	return Py_None;
}

#ifdef FIXME
/*
 *	m_song.enqueue
 *
 */

static 
PyObject *
m_song_enqueue (m_song_object * self, PyObject * args)
{
	m_event_object * event_object;
	int tick = -1;
	
	if (!PyArg_ParseTuple 
		(
			args, 
			"O!|i;needs an "M_EVENT_NAME" and a tick", 
			&m_event_py_type, &event_object, &tick
		)
	) return NULL;

	Py_INCREF (event_object);
	
	self->song->enqueue (event_object->event, tick);
	
	Py_DECREF (event_object);

	Py_INCREF (Py_None);
	return Py_None;
}
#endif


/*
 *	m_song.<start|stop|seek>
 *
 */

static PyObject *
m_song_start (m_song_object * self, PyObject * args)
{
	if (!PyArg_ParseTuple (args, ""))
		return NULL;
		
	self->song->play();

	Py_INCREF (Py_None);
	return Py_None;
}

static PyObject *
m_song_stop (m_song_object * self, PyObject * args)
{
	if (!PyArg_ParseTuple (args, ""))
		return NULL;
		
	self->song->stop();

	Py_INCREF (Py_None);
	return Py_None;
}

static 
PyObject *
m_song_seek (m_song_object * self, PyObject * args)
{
	if (PyTuple_Size (args) <= 0)
	{
		PyErr_SetString (PyExc_ValueError, "need tick or time or (bar, beat, fraction)\n");
		return NULL;
	}

	int bar, beat;
	m_tick tick = TEN_TO_THE_NINTH;
	double time = TEN_TO_THE_NINTH;
	
	bool result;
	
	if (PyArg_ParseTuple (args, "i", &tick))
		result = self->song->seek (time, tick);
	else if (PyArg_ParseTuple (args, "d", &time))
		result = self->song->seek (time, tick);
	else if (PyArg_ParseTuple (args, "iii", &bar, &beat, &tick))
		result = self->song->seek (bar, beat, tick);
	else
	{
		PyErr_SetString (PyExc_ValueError, "need tick or time or (bar, beat, fraction)\n");
		return NULL;
	}

	if (result)
	{
		Py_INCREF (Py_None);
		return Py_None;
	}

	PyErr_SetString (PyExc_ValueError, "illegal song position passed to seek().\n");
	return NULL;
}

/*
 *	m_song.methods
 *
 */

static PyMethodDef
m_song_methods[] = 
{
	{"index", (PyCFunction) m_song_index, METH_VARARGS, 
		"index (part)\n"
		"returns the index of a "M_PART_NAME" in the "M_SONG_NAME"."
	},
	{"add", (PyCFunction) m_song_add, METH_VARARGS, 
		"add (part)\n"
		"adds a "M_PART_NAME" to the "M_SONG_NAME". An error is raised if "
		"part is not a "M_PART_NAME" object."},
	{"remove", (PyCFunction) m_song_remove, METH_VARARGS, 
		"remove (part)\n"
		"removes a "M_PART_NAME" to the "M_SONG_NAME". An error is raised if "
		"the "M_PART_NAME" is not in the "M_SONG_NAME"."},
	{"clear", (PyCFunction) m_song_clear, METH_VARARGS, 
		"clear()\nremoces all parts from a "M_SONG_NAME". No error is raised "
		"the song is already empty."},
	{"read", (PyCFunction) m_song_read, METH_VARARGS, 
		"read (['foo.mid'])\nreads a standard midi file. "
		"The filename is optional and if you do not specify it, the "
		"last used is tried."},
	{"write", (PyCFunction) m_song_write, METH_VARARGS, 
		"write (['copy-of-foo.mid'])\nwrites a standard midi file. "
		"The filename is optional and if you do not specify it, the "
		"last used is tried."},
	{"play", (PyCFunction) m_song_start, METH_VARARGS, 
		"play()\n"
		"starts playing or continues after stopping."},
	{"stop", (PyCFunction) m_song_stop, METH_VARARGS, 
		"stop()\n"
		"stops playing."},
	{"seek", (PyCFunction) m_song_seek, METH_VARARGS, 
		"seek (tick or time)\n"
		"winds the 'MIDI tape' to this tick if you pass in an int, "
		"or to this time if you pass a float value. You can seek while playing."},
	#ifdef FIXME
	{"enqueue", (PyCFunction) m_song_enqueue, METH_VARARGS, 
		"enqueue (event[, tick])\n"
		"puts an "M_EVENT_NAME" on the queue for playback at the "M_EVENT_NAME"s "
		"tick, or at the tick argument if you specify one. If the tick has already "
		"passed, the "M_EVENT_NAME" is sent directly. Raises an error if the "
		"sequencer is not running."},
	#endif
	{NULL, NULL}
};

/*
 *	m_song.getattr
 *
 */

static PyObject *
m_song_getattr (m_song_object * self, char * name)
{
	/* m_file members */
	if (!strcmp (name, "name"))
	{
		if (!self->song->name || !*self->song->name)
			return Py_BuildValue ("s", "");
		return Py_BuildValue ("s", self->song->name);
	}

 	if (!strcmp (name, "division"))
		return Py_BuildValue ("i", self->song->division);

	/* m_song members */
 	if (!strcmp (name, "playing"))
		return Py_BuildValue ("i", self->song->playing);

 	if (!strcmp (name, "tick"))
		return Py_BuildValue ("i", self->song->tick.current);

 	if (!strcmp (name, "time"))
		return Py_BuildValue ("d", self->song->time.current);

 	if (!strcmp (name, "loop"))
		return Py_BuildValue ("ii", self->song->loop.start, self->song->loop.end);

	if (!strcmp (name, "beat"))
		return Py_BuildValue (
			"(iii)", 
				self->song->beat.bar,
				self->song->beat.beat,
				self->song->beat.fraction
		);

 	if (!strcmp (name, "numerator"))
		return Py_BuildValue ("i", self->song->beat.numerator);

 	if (!strcmp (name, "denominator"))
		return Py_BuildValue ("i", self->song->beat.denominator);

 	if (!strcmp (name, "ticks_per_beat"))
		return Py_BuildValue ("i", self->song->beat.tick.per_beat);

 	if (!strcmp (name, "click"))
		return (PyObject *) m_event_new_from_event (&self->song->beat.click);

	if (!strcmp (name, "__members__"))
	{
		PyObject * result = PyList_New (11);
		if (result)
		{
			PyList_SetItem (result, 0, PyString_FromString ("name"));
			PyList_SetItem (result, 1, PyString_FromString ("division"));
			PyList_SetItem (result, 2, PyString_FromString ("playing"));
			PyList_SetItem (result, 3, PyString_FromString ("tick"));
			PyList_SetItem (result, 4, PyString_FromString ("time"));
			PyList_SetItem (result, 5, PyString_FromString ("loop"));
			PyList_SetItem (result, 6, PyString_FromString ("beat"));
			PyList_SetItem (result, 7, PyString_FromString ("numerator"));
			PyList_SetItem (result, 8, PyString_FromString ("denominator"));
			PyList_SetItem (result, 9, PyString_FromString ("ticks_per_beat"));
			PyList_SetItem (result, 10, PyString_FromString ("click"));
		
			if (PyErr_Occurred()) 
			{
				Py_DECREF (result);
				result = NULL;
			}
		}
		return result;
	}
	
	return Py_FindMethod (m_song_methods, (PyObject *) self, name);
}

/*
 *	m_song.setattr
 *
 */

static int
m_song_setattr (m_song_object * self, char * name, PyObject * value)
{
	if (!value) 
	{
		PyErr_SetString (PyExc_TypeError, "attribute deletion is not supported");
		return -1;
	}
	
	if (!strcmp (name, "playing"))
	{
		if (!PyInt_Check (value))
		{
			PyErr_SetString (PyExc_TypeError, "need an int.");
			return -1;
		}

		if (PyInt_AS_LONG (value)) self->song->play();
		else self->song->stop();

		return 0;
	}

	if (!strcmp (name, "name"))
	{
		char * filename = PyString_AsString (value);
		if (filename)
			self->song->set_name (filename);
		return 0;
	}

 	if (!strcmp (name, "division"))
	{
		int division = PyInt_AsLong (value);
		if (division <= 0 || division > 0x7FFF)
		{
			PyErr_SetString (PyExc_TypeError, "0 < division < 0x7FFF");
			return -1;
		}

		self->song->division = division;
		return 0;
	}

	/* set the loop from (ii) */
	if (!strcmp (name, "loop"))
	{
		m_tick start, end;
		if (!PyArg_ParseTuple (value, "ii;loop start and end", &start, &end))
			return -1;

		if (start < 0 || end < 0)
		{
			PyErr_SetString (PyExc_ValueError, 
				"neither loop start nor end ticks can be < 0.");
			return -1;
		}
		self->song->loop.start = start;
		self->song->loop.end = end;
		return 0;
	}

	/* set the click event */
 	if (!strcmp (name, "click"))
	{
		if (value == Py_None)
		{
			self->song->beat.click.set_velocity_on (0);
			return 0;
		}

		if (PyObject_Type (value) != (PyObject *) &m_event_py_type)
		{
			PyErr_SetString (PyExc_TypeError, "give me an "M_EVENT_NAME".");
			return -1;
		}

		self->song->beat.click.copy (((m_event_object *) value)->event);
		return 0;
	}
	
	/* tick and time */
	if (!strcmp (name, "tick"))
	{
		m_tick tick = PyInt_AsLong (value);

		if (PyErr_Occurred())
			return -1;
			
		if (self->song->seek (TEN_TO_THE_NINTH, tick))
			return 0;
			
		PyErr_SetString (PyExc_ValueError, "invalid tick.");
		return -1;
	}

	if (!strcmp (name, "time"))
	{
		double tim = PyFloat_AsDouble (value);

		if (PyErr_Occurred())
			return -1;
			
		if (self->song->seek (tim, TEN_TO_THE_NINTH))
			return 0;
			
		PyErr_SetString (PyExc_ValueError, "invalid time.");
		return -1;
	}

	PyErr_SetString (PyExc_AttributeError, name);
	return -1;
}


/*
 *	m_song as sequence
 *
 */

static int
m_song_len (m_song_object * song_object)
{
	return song_object->song->parts.count;
}

/*
 *	= m_song[i]
 *
 */

static m_part_object *
m_song_item (m_song_object * self, int i)
{
	if (i < 0 || i >= self->song->parts.count)
	{
		PyErr_SetString (PyExc_IndexError, "part index is out of range.");
		return NULL;
	}
	
	m_part_object * part_object = m_part_new_from_song (self, i);
	if (!part_object)
	{
		PyErr_SetString (PyExc_MemoryError, "failed to create a m_part object.");
		return NULL;
	}

	return part_object;
}
	
/*
 *	m_song[i] =
 *
 */

static int
m_song_ass_item (m_song_object * self, int i, PyObject * song_object)
{
	if (i < 0 || i >= self->song->parts.count)
	{
		PyErr_SetString (PyExc_IndexError, "part index out of range.");
		return -1;
	}

	if (song_object == NULL)
	{
		self->song->remove_part (self->song->part(i));
		return 0;
	}

	PyErr_SetString (PyExc_NotImplementedError, "sorry I can't do that.");
	return -1;
}


/* 
  repr && print 
 */

static PyObject *
m_song_repr (m_song_object * self)
{
	c_char_buffer buffer;
	
	#if 1
	buffer << M_SONG_NAME << " ('" << self->song->name << "')";
	#else
	self->song->repr (&buffer, '\n');
	#endif
	
	return PyString_FromStringAndSize (buffer.items, buffer.count);
}

static int
m_song_print (m_song_object * self, FILE * fp, int flags)
{
	c_char_buffer buffer;
	
	#if 1
	buffer << M_SONG_NAME << " ('" << self->song->name << "')";
	#else
	self->song->repr (&buffer, '\n');
	#endif

	buffer >> fp;

	return 0;
}

/*
 *	m_song as python type
 *
 */

static 
PySequenceMethods 
m_song_as_sequence = 
{
	(inquiry) m_song_len, //sq_length
	(binaryfunc) 0, //sq_concat
	(intargfunc) 0, //sq_repeat
	(intargfunc) m_song_item, //sq_item
	(intintargfunc) 0, //sq_slice
	(intobjargproc) m_song_ass_item, //sq_ass_item
	(intintobjargproc) 0, //sq_ass_slice
	(objobjproc) 0, //sq_contains
};

PyTypeObject 
m_song_py_type =
{
	PyObject_HEAD_INIT (&PyType_Type)
	0, // ob_size 
	M_SONG_NAME,
	sizeof (m_song_object),
	0, // tp_itemsize 

	// methods 
	
	(destructor) m_song_del,
	(printfunc) m_song_print,
	(getattrfunc) m_song_getattr,
	(setattrfunc) m_song_setattr,
	(cmpfunc) 0,
	(reprfunc) m_song_repr,
	0, // tp_as_number 
	&m_song_as_sequence, // tp_as_sequence 
	0, // tp_as_mapping 
	(hashfunc) 0,
	(ternaryfunc) 0,
	(reprfunc) 0,

	// reserved ... 
	
	0,0,0,0,

	// __doc__ 
	
	"A "M_SONG_NAME" is an unsliceable sequence of midi "M_PART_NAME"s (at "
	"least to midithing). The first "M_PART_NAME" should contain "M_META_NAME" "
	M_EVENT_NAME"s specifying the Time and Tempo measures. The "M_IO_NAME" "
	" object expects this, and all midi software out there, too."
};

/*
 *	and the constructor.
 *
 */

m_song_object *
module_m_song_new (PyObject * self, PyObject * args)
{
	char * filename = NULL;

	if (!PyArg_ParseTuple (args, "|s", &filename))
		return NULL;
		
	m_song * song = new m_song;

	if (filename && !song->read (filename))
	{
		PyErr_SetString (PyExc_IOError, song->get_error());
		delete song;
		return NULL;
	}

	return m_song_new_from_song (song);
}








