/*
 *	python-types.h
 *
 */

#ifndef _PYTHON_TYPES_H_
#define _PYTHON_TYPES_H_

#define DBG_REF 0

#include <pthread.h>
#include <Python.h>

#include "midi-event.h"
#include "midi-part.h"
#include "midi-file.h"
#include "midi-meta-types.h"
#include "midi-io.h"
#include "midi-clock.h"


#define M_NOTE_NAME_NAME	"note_name"


class py_callback
:
public m_connector
{
	public:
		PyThreadState * tstate;
		PyObject *		callback;
		
									py_callback();
								 ~py_callback();
		
		virtual bool	event_in (m_event * e, m_event_stamp * = 0);

		bool					set_callback (PyObject * _callback);
		bool					call (char * fmt, ...);
		bool					call_with_tuple (PyObject *);
};






/*
 *	Song / file
 *
 */

typedef struct 
{
	PyObject_HEAD
	m_song * song;
} 
m_song_object;

extern PyTypeObject m_song_py_type;

#define m_song_Check(op) ((op)->ob_type == &m_song_py_type)

extern m_song_object *
m_song_new_from_song (m_song * song);

extern m_song_object *
module_m_song_new (PyObject * self, PyObject * args);








/*
 *	Part
 *
 */

typedef struct 
{
	PyObject_HEAD
	m_part * part;
} 
m_part_object;

extern PyTypeObject m_part_py_type;

#define m_part_Check(op) ((op)->ob_type == &m_part_py_type)

extern m_part_object *
m_part_new_from_song (m_song_object * song_object, int i);

extern m_part_object *
module_m_part_new (PyObject * self, PyObject * args, PyObject * keywds);








/*
 *	Event
 *
 */

typedef struct 
{
	PyObject_HEAD
	m_event * event;
} 
m_event_object;

extern PyTypeObject m_event_py_type;

#define m_event_Check(op) ((op)->ob_type == &m_event_py_type)

extern m_event_object *
m_event_new();
	
extern m_event_object *
m_event_new_from_event (m_event *);
	
extern m_event_object *
module_m_event_new (PyObject * self, PyObject * args, PyObject * keywds);

extern PyObject *
m_event_get_value (m_event * e, char * name);

extern bool
m_event_set_value (m_event * e, char * name, PyObject * py_value);

/* constructors */
extern m_event_object *
module_m_note_new (PyObject * self, PyObject * args, PyObject * keywds);

extern m_event_object *
module_m_note_off_new (PyObject * self, PyObject * args, PyObject * keywds);

extern m_event_object *
module_m_note_touch_new (PyObject * self, PyObject * args, PyObject * keywds);

extern m_event_object *
module_m_controller_new (PyObject * self, PyObject * args, PyObject * keywds);

extern m_event_object *
module_m_program_new (PyObject * self, PyObject * args, PyObject * keywds);

extern m_event_object *
module_m_channel_touch_new (PyObject * self, PyObject * args, PyObject * keywds);

extern m_event_object *
module_m_pitch_new (PyObject * self, PyObject * args, PyObject * keywds);

extern m_event_object *
module_m_sysex_new (PyObject * self, PyObject * args, PyObject * keywds);

extern m_event_object *
module_m_meta_new (PyObject * self, PyObject * args, PyObject * keywds);







/*
 *  Port
 *  
 */

typedef struct 
{
	PyObject_HEAD
	m_connector * port;
} 
m_port_object;

extern PyTypeObject m_port_py_type;

#define m_port_Check(op) ((op)->ob_type == &m_port_py_type)

extern m_port_object *
m_port_new_from_port (m_connector * port);

extern m_port_object *
module_m_port_new (PyObject * self, PyObject * args);

extern PyObject *
module_m_loopback_new (PyObject * self, PyObject * args);
	
extern PyObject *
module_m_ascii_port_new (PyObject * self, PyObject * args);
	
#ifdef WITH_ALSA

extern m_port_object *
module_m_alsa_port_new (PyObject * self, PyObject * args);

#endif /* WITH_ALSA */







/*
 * 	IO
 *
 */

typedef struct 
{
	PyObject_HEAD
	m_io * io;
	py_callback * on_input;
} m_io_object;

extern PyTypeObject m_io_py_type;

#define m_io_Check(op) ((op)->ob_type == &m_io_py_type)

extern m_io_object *
module_m_io_new (PyObject * self, PyObject * args);






/*
 * 	Sync
 *
 */

typedef struct
{
	PyObject_HEAD
	m_io * io;
	m_sync * sync;
} m_sync_object;

extern PyTypeObject m_sync_py_type;

#define m_sync_Check(op) ((op)->ob_type == &m_sync_py_type)

extern m_sync_object *
m_sync_new_from_io (m_io *);




/*
 * 	Clock
 *
 */

typedef struct
{
	PyObject_HEAD
	m_clock * clock;
} m_clock_object;

extern PyTypeObject m_clock_py_type;

#define m_clock_Check(op) ((op)->ob_type == &m_clock_py_type)

extern m_clock_object *
m_clock_new_from_clock (m_clock *);

extern m_clock_object *
module_m_clock_new (PyObject * self, PyObject * args);



/*
 *	is_a_connector
 *
 */

inline m_connector *
m_connector_Cast (PyObject * object)
{
	if (m_port_Check (object))
		return ((m_port_object *) object)->port;
	if (m_io_Check (object))
		return ((m_io_object *) object)->io;
	if (m_part_Check (object))
		return ((m_part_object *) object)->part;
	return NULL;
}

#endif /* _PYTHON_TYPES_ */
